/** deadlock.c
 * This test file demonstrates deadlock detection by GMC.
 * Running it in interpretation mode will succeed (main thread will first
 * lock and unlock both mutexes, then t1 will do the same in different order).
 * In model checker mode, all of the orders of accessess will be tried out,
 * resulting in detecting a deadlock.
 */

#include <pthread.h>
#include <stdio.h>

pthread_mutex_t m1, m2;

void* thread1(void* data)
{
	pthread_mutex_lock(&m2);
	//printf("t1 locked m2\n");
	pthread_mutex_lock(&m1);
	//printf("t1 locked m1\n");
	pthread_mutex_unlock(&m1);
	//printf("t1 unlocked m1\n");
	pthread_mutex_unlock(&m2);
	//printf("t1 unlocked m2\n");
}

int main()
{
	pthread_t t1;
	pthread_mutex_init(&m1, NULL);
	pthread_mutex_init(&m2, NULL);
	pthread_create(&t1, NULL, thread1, NULL);
	pthread_mutex_lock(&m1);
	//printf("main locked m1\n");
	pthread_mutex_lock(&m2);
	//printf("main locked m2\n");
	pthread_mutex_unlock(&m2);
	//printf("main unlocked m2\n");
	pthread_mutex_unlock(&m1);
	//printf("main unlocked m1\n");
	pthread_join(t1, NULL);
	printf("%s\n", "OK!");
	return 0;
}
