// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file Error.cpp
 * Definitions for (some) error reporting functions.
 */
#include "Error.h"

#include <cstdarg>
#include <cstdio>
#include <cstdlib>


static void vdo_error (FILE* f, const char* error_type, const char* format, 
                       const char* file, const char* function, int line, va_list ap) ATTRIBUTE_NORETURN;
void vdo_ni_error (const char* format, const char* file, const char* function, 
                   int line, va_list ap) ATTRIBUTE_NORETURN;
void vrt_error (const char* format, va_list ap) ATTRIBUTE_NORETURN;


static void
vdo_error (FILE* f, const char* error_type, const char* format, 
           const char* file, const char* function, int line, va_list ap)
{
  fprintf (f, "error in %s, at %s:%d: %s: ", function, file, line, error_type);
  vfprintf (f, format, ap);
  fputc ('\n', f);
  abort ();
}


void 
vdo_ni_error (const char* format, const char* file, const char* function, 
              int line, va_list ap)
{
  vdo_error (stderr, "not implemented", format, file, function, line, ap);
}


void 
do_ni_error (const char* file, const char* function, int line, 
             const char* format, ...) 
{
  va_list ap;
  va_start (ap, format);
  vdo_ni_error (format, file, function, line, ap);
  va_end (ap);
}


void 
do_ni_assert (bool condition, const char* file, const char* function, int line, 
              const char* format, ...) 
{
  if (!condition)
    {
      va_list ap;

      va_start (ap, format);
      vdo_ni_error (format, file, function, line, ap);
      va_end (ap);
    }
}


void 
vrt_error (const char* format, va_list ap)
{
  vdo_error (stderr, "run-time error", format, "", "", 0, ap);
}


void 
rt_error (const char* format, ...)
{
  va_list ap;

  va_start (ap, format);
  vrt_error (format, ap);
  va_end (ap);
}

void 
rt_assert (bool condition, const char* format, ...)
{
   if (!condition) {
      va_list ap;

      va_start (ap, format);
      vrt_error (format, ap);
      va_end (ap);
    }
}



