// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file Main.cpp
 * Contains the main function of the program.
 */

#include "int/Interpret.h"
#include "int/ModelChecker.h"

#include <iostream>
#include <unistd.h>
#include <cstring>
#include <libgen.h>

using namespace ginterpreter;

#define USAGE_STR " -i|-m [--dump] source1 [source2 ...]"
#define INTERPRET_OPTION "-i"
#define MODELCHECK_OPTION "-m"

/** @mainpage
 *
 * This is programmer's documentation of GIMPLE Model Checker (GMC)
 * and the modules it uses : Memory Module, GIMPLE Interpreter
 * and GIMPLE Iterator.
 *
 * The code of the Memory Module is in @c mmodule namespace,
 * the code of GIMPLE Iterator module is in @c giterator namespace
 * and the code of the GIMPLE Interpreter is in @c ginterpreter namespace.
 * GIMPLE Iterator and Memory Module are independent of the GIMPLE Interpreter
 * and can be easily used separately.
 *
 * The documentation of GIMPLE Interpreter is partially created with
 * a use of documentation from gcc's tree.def and gimple.def files
 * and the documentation available at http://gcc.gnu.org/onlinedocs/gccint
 */

void
reportUsage(const char* progName)
{
	std::cout << "usage: "<< progName << USAGE_STR << std::endl;
	std::cerr << "usage: "<< progName << USAGE_STR << std::endl;
}

int
main(int argc, char** argv)
{
	// flags not used at the moment
	struct interpreter_flags iflags;

	iflags.backtrack = 0;
	iflags.backtrack_fine = 0;
	iflags.state_reentry_check = 0;
	iflags.state_reentry_check_fine = 0;

	if (argc < 3)
	{
		reportUsage(basename(argv[0]));
		return -1;
	}

	if (strcmp(argv[1], INTERPRET_OPTION) == 0)
		return doInterpret(iflags, argv[2]);
	else if (strcmp(argv[1], MODELCHECK_OPTION) == 0)
		return doModelCheck(argv[2]);
	else
	{
		reportUsage(basename(argv[0]));
		return -1;
	}
}
