// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file AddrOperand.h
 * Definition of an address operand.
 */
#ifndef GIT_ADDR_OPERAND_H
#define GIT_ADDR_OPERAND_H

#include "DecoratingOperand.h"

#include <boost/serialization/serialization.hpp>

namespace giterator {

/**
 * The operand that represents an address of another operand.
 * That is '&' in C, and an ADDR_EXPR in GIMPLE.
 * These operands have pointer type. The sub operand
 * may be another expression or a declaration.
 *
 * @see create_git_operand_from_tree to see how address
 * operands get created from GIMPLE operands
 */
class AddrOperand : public detail::DecoratingOperand {
  public:
	/**
	 * Constructor, creates new address operand.
	 * Empty constructor exists for serialization purposes,
     * use nonempty for normal use.
	 */
	AddrOperand () {}
    /**
     * Constructor, creates new address operand.
     * 
     * @param type the type of the pointer
     * @param oper the operand which address this operand represents
     */
    AddrOperand (const Type* type, std::auto_ptr<Operand> oper) : 
        detail::DecoratingOperand (type, oper) {}
       
    virtual void accept (Visitor& v) const
    { v.visit (*this); }

    friend class boost::serialization::access;
	template<class Archive>
	void serialize(Archive & ar, const unsigned int)
	{
		boost::serialization::void_cast_register(static_cast<AddrOperand *>(NULL), static_cast<Operand *>(NULL));
		ar & boost::serialization::base_object<DecoratingOperand>(*this);
	}
};

}

#endif /* GIT_ADDR_OPERAND_H */

