// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file BoolType.h
 * Class representing boolean type.
 */
#ifndef GIT_BOOL_TYPE_H
#define GIT_BOOL_TYPE_H

#include "AbstractType.h"

#include <typeinfo>
#include <boost/serialization/serialization.hpp>


namespace giterator { 
    
/**
 * Class representing boolean type.
 * This type gets created from GIMPLE @c BOOL_TYPE
 * (_Bool in C).
 *
 * @see create_git_type_from_tree to see how boolean types
 * get created from GIMPLE types
 */
class BoolType : public detail::AbstractType {
  public:
	/**
	 * Constructor, creates a new boolean type.
	 * Empty constructor exists for serialization purposes,
	 * use nonempty for normal use.
	 */
	BoolType() : detail::AbstractType() {}

	/**
	 * Constructor, creates a new boolean type.
	 *
	 * @param name name of this boolean type
	 * @param size of this type in bits
	 */
    BoolType (const std::string& name, size_t size) 
        : detail::AbstractType (name, size) {}

    /**
     * Compare this type to another type for compatibility.
     * Apart from any boolean type, integer type also passes
     * this function with true.
     *
     * @param type to be compared to this type
     * @returns true if the types are the same (compatibile),
     * false otherwise
     */
    bool isSame (const Type& type) const
    { 	// TODO:
    	return true;
    	//return typeid (type) == typeid (BoolType) || typeid (type) == typeid (IntegerType); }
    }

    friend class boost::serialization::access;
	template<class Archive>
	void serialize(Archive & ar, const unsigned int)
	{
		boost::serialization::void_cast_register(static_cast<BoolType *>(NULL), static_cast<TreeProduct *>(NULL));
		boost::serialization::void_cast_register(static_cast<BoolType *>(NULL), static_cast<Type *>(NULL));
		ar & boost::serialization::base_object<AbstractType>(*this);
	}
};

}

#endif /* GIT_BOOL_TYPE_H */
