// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file Constant.h
 * Interface for classes representing constants.
 */
#ifndef GIT_CONSTANT_H
#define GIT_CONSTANT_H

#include "TreeProduct.h"

namespace giterator { 
    
class IntegerConstant;
class EnumeralConstant;
class RealConstant;
class StringConstant;
class Type;
    
/**
 * Interface for classes representing constants.
 * Constants get created from GIMPLE @c *_CST nodes.
 * Each of the constants store its type.
 *
 * @see create_git_constant_from_tree() in gimplexx_bridge.c
 * to see how constants get created from GIMPLE constants.
 */
class Constant : public detail::TreeProduct {
  public:
    /**
     * Visitor class for sub-classes of @c Constant class, that are meant to be used 
     * by the user.
     */
    struct Visitor {
        virtual ~Visitor () {}
        
        virtual void visit (const IntegerConstant& cst) = 0;
        virtual void visit (const EnumeralConstant& cst) = 0;
        virtual void visit (const RealConstant& cst) = 0;
        virtual void visit (const StringConstant& cst) = 0;
    };
    
  public:
    /**
	 * Destructor.
	 */
    virtual ~Constant () {}
      
    /**
     * Returns the type of this constant. 
     *
     * @return type of this constant
     */
    virtual const Type* getType () const = 0;

    /**
     * Calls @c visitor.accept().
     * Classes implementing this interface implement this method to call @c visitor.accept()
     * with @c *this as a parameter.
     * 
     * @param visitor visitor on which to call the @c accept method
     */
    virtual void accept (Visitor& visitor) const = 0;
};

}


#endif /* GIT_CONSTANT_H */
