// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file ConstructorOperand.h
 * Base class for constructor operands.
 */
#ifndef GIT_CONSTRUCTOR_OPERAND_H
#define GIT_CONSTRUCTOR_OPERAND_H

#include "TypeOperand.h"

namespace giterator
{
	/**
	 * Base class for constructor operands.
	 * Constructor operands are operands representing
	 * constructors (initializers) of data structures
	 * (arrays, structures), for example:
	 * <code> int arr[3] = { 1, 2, 3}; </code>
	 * is an array declaration with constructor.
	 * Concrete initializers are subclasses of this class.
	 *
	 * Constructor operands get created from GIMPLE
	 * @c CONSTRUCTOR nodes.
	 *
	 * @see giterator::RecordConstructorOperand,
	 *  giterator::ArrayConstructorOperand
	 */
	class ConstructorOperand : public detail::TypeOperand
	{
	public:
		/**
		 * Constructor, creates a new constructor operand.
		 * Empty constructor exists for serialization purposes,
		 * use nonempty for normal use.
		 */
		ConstructorOperand() : TypeOperand() {}
		/**
		 * Constructor, creates a new constructor operand.
		 *
		 * @param type the constructed type.
		 */
		ConstructorOperand(const Type* type) : TypeOperand(type) {}
	};
}

#endif /* GIT_CONSTRUCTOR_OPERAND_H */
