// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file EnumeralConstant.h
 * Class representing an enumeral constant.
 */
#ifndef GIT_ENUMERAL_CONSTANT_H
#define GIT_ENUMERAL_CONSTANT_H

#include "IntegerConstant.h"
#include "Type.h"
#include "Serialization.h"

namespace giterator {

/**
 * Class representing an enumeral constant.
 * Enumeral constants in GIMPLE are represented as
 * @c INTEGER_CST with @c ENUMERAL_TYPE.
 *
 * @see giterator::IntegerConstant
 * @see create_git_constant_from_tree in gimplexx-bridge.c to
 * see how enumeral constants get created from GIMPLE constants
 */
class EnumeralConstant : public IntegerConstant {
  public:
	/**
	 * Constructor, creates a new enumeration constant.
	 * Empty constructor exists for serialization purposes,
	 * use nonempty for normal use.
	 */
    EnumeralConstant() {}
    /**
     * Constructor, creates a new enumeration constant.
     *
     * @param type the type of the constant
     * @param value raw value of the constant. Same as with @c IntegerType.
     */
    EnumeralConstant (const Type* type, uint64_t value)
        : IntegerConstant(type, value) {}

    virtual void accept (Visitor& visitor) const
    { visitor.visit (*this); }

	template<class Archive>
	void serialize(Archive & ar, const unsigned int version)
	{
		boost::serialization::void_cast_register(static_cast<EnumeralConstant *>(NULL), static_cast<TreeProduct *>(NULL));
		boost::serialization::void_cast_register(static_cast<EnumeralConstant *>(NULL), static_cast<Constant *>(NULL));
		boost::serialization::split_member(ar, *this, version);
	}
};

}


#endif /* GIT_ENUMERAL_CONSTANT_H */
