// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file FieldDecl.h
 * Class representing declaration of a field (of struct/union).
 */
#ifndef GIT_FIELD_DECL_H
#define GIT_FIELD_DECL_H

#include "OffsetDecl.h"
#include "Type.h"

#include <boost/serialization/serialization.hpp>

namespace giterator {

/**
 * Class representing declaration of a field (of stuct/union).
 * Field decls are non-static data members. In GIMPLE++,
 * they are equivalent to variable declarations, except they
 * are accessible through record types.
 * Field declarations get created from GIMPLE @c FIELD_DECL
 * nodes.
 *
 * @see giterator::RecordType, giterator::RecordDecl
 * @see create_git_decl_from_tree in gimplexx-bridge.c to see how
 * declarations of struct/union fields get created from
 * GIMPLE declarations
 */
class FieldDecl : public OffsetDecl {
  public:
	/**
	 * Constructor, creates a new field declaration.
	 * Empty constructor exists for serialization purposes,
	 * use nonempty for normal use.
	 */
	FieldDecl () : OffsetDecl() {}

	/**
	 * Constructor, creates a new field declaration.
	 *
     * @param uid unique ID of the new declaration
     * @param name name of the new declaration
     * @param type type of the new declaration
     */
    FieldDecl (unsigned int uid, const std::string& name, const Type* type) :
                OffsetDecl (uid, name, type) {}

    void accept (Visitor& visitor) const
    { visitor.visit (*this); }

    friend class boost::serialization::access;
	template<class Archive>
	void serialize(Archive & ar, const unsigned int version)
	{
		boost::serialization::void_cast_register(static_cast<FieldDecl *>(NULL), static_cast<TreeProduct *>(NULL));
		boost::serialization::void_cast_register(static_cast<FieldDecl *>(NULL), static_cast<Decl *>(NULL));
		ar & boost::serialization::base_object<OffsetDecl>(*this);
	}
};

}

#endif /* GIT_FIELD_DECL_H */
