// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file FunctionInfo.h
 * Interpreted builtin and standard functions data needed.
 */
#ifndef GIT_FUNCTION_INFO_H
#define GIT_FUNCTION_INFO_H

#ifdef __cplusplus
extern "C" {
namespace giterator
{
#endif

/** @name Names of builtin (standard) function in GIMPLE
 * Function names, that may appear in GIMPLE for functions
 * simulated in GIMPLE Interpreter.
 */
/*@{*/
#define BUILT_IN_PRINTF_NAME "printf"
#define BUILT_IN_PRINTF_NAME2 "__builtin_printf"
#define BUILT_IN_PUTS_NAME "puts"
#define BUILT_IN_PUTS_NAME2 "__builtin_puts"
#define BUILT_IN_PUTCHAR_NAME "putchar"
#define BUILT_IN_PUTCHAR_NAME2 "__builtin_putchar"
#define BUILT_IN_SCANF_NAME "scanf"
#define BUILT_IN_SCANF_NAME2 "__builtin_scanf"
#define BUILT_IN_MALLOC_NAME "malloc"
#define BUILT_IN_MALLOC_NAME2 "__builtin_malloc"
#define BUILT_IN_FREE_NAME "free"
#define BUILT_IN_FREE_NAME2 "__builtin_free"
#define BUILT_IN_ABORT_NAME "abort"
#define BUILT_IN_ABS_NAME "abs"
#define BUILT_IN_LABS_NAME "labs"
#define BUILT_IN_LLABS_NAME "llabs"
#define BUILT_IN_ATOF_NAME "atof"
#define BUILT_IN_ATOI_NAME "atoi"
#define BUILT_IN_ATOL_NAME "atol"
#define BUILT_IN_ATOLL_NAME "atoll"
#define BUILT_IN_EXIT_NAME "exit"
#define BUILT_IN_RAND_NAME "rand"
#define BUILT_IN_RANDOM_NAME "random"
#define BUILT_IN_SRAND_NAME "srand"
#define BUILT_IN_SRANDOM_NAME "srandom"
#define BUILT_IN_STRTOD_NAME "strtod"
#define BUILT_IN_STRTOLD_NAME "strtold"
#define BUILT_IN_STRTOF_NAME "strtof"
#define BUILT_IN_STRTOL_NAME "strtol"
#define BUILT_IN_STRTOLL_NAME "strtoll"
#define BUILT_IN_STRTOUL_NAME "strtoul"
#define BUILT_IN_STRTOULL_NAME "strtoull"
#define BUILT_IN_MEMCPY_NAME "memcpy"
#define BUILT_IN_MEMCPY_NAME2 "__builtin_memcpy"
#define BUILT_IN_MEMMOVE_NAME "memmove"
#define BUILT_IN_MEMMOVE_NAME2 "__builtin_memmove"
#define BUILT_IN_STRCAT_NAME "strcat"
#define BUILT_IN_STRCAT_NAME2 "__builtin_strcat"
#define BUILT_IN_STRNCAT_NAME "strncat"
#define BUILT_IN_STRNCAT_NAME2 "__builtin_strncat"
#define BUILT_IN_STRCPY_NAME "strcpy"
#define BUILT_IN_STRCPY_NAME2 "__builtin_strcpy"
#define BUILT_IN_STRCNPY_NAME "strncpy"
#define BUILT_IN_STRCNPY_NAME2 "__builtin_strncpy"
#define BUILT_IN_STRCMP_NAME "strcmp"
#define BUILT_IN_STRCMP_NAME2 "__builtin_strcmp"
#define BUILT_IN_STRNCMP_NAME "strncmp"
#define BUILT_IN_STRNCMP_NAME2 "__builtin_strncmp"
#define BUILT_IN_STRLEN_NAME "strlen"
#define BUILT_IN_STRLEN_NAME2 "__builtin_strlen"
#define BUILT_IN_ASSERT_NAME "assert"
#define BUILT_IN_ASSERT_NAME2 "__assert"
#define BUILT_IN_ASSERT_NAME3 "__assert_fail"
#define BUILT_IN_SLEEP_NAME "sleep"
/*@}*/

/**
 * Enumeration determining kind of function.
 * There is an enumeration constant for each supported function
 * in GIMPLE Interpreter and one for unsupported functions.
 *
 * @see BuiltinFunctions.h
 */
enum function_code
{
	NOT_BUILT_IN_FUNC,			/**< Not a built-in (standard) function, or unsupported builtin (standard) function. */
	GCC_BUILT_IN_PRINTF,		/**< Builtin printf() (<stdio.h>) */
	GCC_BUILT_IN_PUTS,			/**< Builtin puts() (<stdio.h>) */
	GCC_BUILT_IN_PUTCHAR,		/**< Builtin putchar() (<stdio.h>) */
	GCC_BUILT_IN_SCANF,			/**< Builtin scanf() (<stdio.h>) */
	GCC_BUILT_IN_MALLOC,		/**< Builtin malloc() (<stdlib.h>) */
	GCC_BUILT_IN_FREE,			/**< Builtin free() (<stdlib.h>) */
	GCC_BUILT_IN_ABORT,			/**< Builtin abort() (<stdlib.h>) */
	GCC_BUILT_IN_ABS,			/**< Builtin abs() (<stdlib.h>) */
	GCC_BUILT_IN_LABS,			/**< Builtin labs() (<stdlib.h>) */
	GCC_BUILT_IN_LLABS,			/**< Builtin llabs() (<stdlib.h>) */
	GCC_BUILT_IN_ATOF,			/**< Standard atof() (<stdlib.h>) */
	GCC_BUILT_IN_ATOI,			/**< Standard atoi() (<stdlib.h>) */
	GCC_BUILT_IN_ATOL,			/**< Standard atol() (<stdlib.h>) */
	GCC_BUILT_IN_ATOLL,			/**< Standard atoll() (<stdlib.h>) */
	GCC_BUILT_IN_EXIT,			/**< Builtin exit() (<stdlib.h>) */
	GCC_BUILT_IN_RAND,			/**< Standard rand() (<stdlib.h>) */
	GCC_BUILT_IN_RANDOM,		/**< Standard random() (<stdlib.h>) */
	GCC_BUILT_IN_SRAND,			/**< Standard srand() (<stdlib.h>) */
	GCC_BUILT_IN_SRANDOM,		/**< Standard srandom() (<stdlib.h>) */
	GCC_BUILT_IN_STRTOD,		/**< Standard strtod() (<stdlib.h>) */
	GCC_BUILT_IN_STRTOLD,		/**< Standard strtold() (<stdlib.h>) */
	GCC_BUILT_IN_STRTOF,		/**< Standard strtof() (<stdlib.h>) */
	GCC_BUILT_IN_STRTOL,		/**< Standard strtol() (<stdlib.h>) */
	GCC_BUILT_IN_STRTOLL,		/**< Standard strtoll() (<stdlib.h>) */
	GCC_BUILT_IN_STRTOUL,		/**< Standard strtoul() (<stdlib.h>) */
	GCC_BUILT_IN_STRTOULL,		/**< Standard strtoull() (<stdlib.h>) */
	GCC_BUILT_IN_MEMCPY,		/**< Builtin memcpy() (<string.h>) */
	GCC_BUILT_IN_MEMMOVE,		/**< Builtin memmove() (<string.h>) */
	GCC_BUILT_IN_STRCAT,		/**< Builtin strcat() (<string.h>) */
	GCC_BUILT_IN_STRNCAT,		/**< Builtin strncat() (<string.h>) */
	GCC_BUILT_IN_STRCPY,		/**< Builtin strcpy() (<string.h>) */
	GCC_BUILT_IN_STRCNPY,		/**< Builtin strcnpy() (<string.h>) */
	GCC_BUILT_IN_STRCMP,		/**< Builtin strcmp() (<string.h>) */
	GCC_BUILT_IN_STRNCMP,		/**< Builtin strncmp() (<string.h>) */
	GCC_BUILT_IN_STRLEN,		/**< Builtin strlen() (<string.h>) */
	GCC_BUILT_IN_ASSERT,		/**< assert - technically a macro (<assert.h>)*/
	GCC_BUILT_IN_SLEEP			/**< Standard sleep() (<unistd.h>) */
};

enum thread_function_code
{
	THREAD_CREATE,				/**< pthred_create (<pthread.h>)(or equivalent) */
	THREAD_EXIT,				/**< pthred_exit (<pthread.h>)(or equivalent) */
	THREAD_JOIN,				/**< pthred_join (<pthread.h>)(or equivalent) */
	THREAD_MUTEX_INIT,			/**< pthred_mutex_init (<pthread.h>)(or equivalent) */
	THREAD_MUTEX_DESTROY,		/**< pthred_mutex_destroy (<pthread.h>)(or equivalent) */
	THREAD_MUTEX_LOCK,			/**< pthred_mutex_lock (<pthread.h>)(or equivalent) */
	THREAD_MUTEX_TRYLOCK,		/**< pthred_mutex_trylock (<pthread.h>)(or equivalent) */
	THREAD_MUTEX_UNLOCK,		/**< pthred_mutex_unlock (<pthread.h>)(or equivalent) */
	THREAD_COND_INIT,			/**< pthred_cond_init (<pthread.h>)(or equivalent) */
	THREAD_COND_DESTROY,		/**< pthred_cond_destroy (<pthread.h>)(or equivalent) */
	THREAD_COND_SIGNAL,			/**< pthred_cond_signal (<pthread.h>)(or equivalent) */
	THREAD_COND_WAIT,			/**< pthred_cond_wait (<pthread.h>)(or equivalent) */
	THREAD_SLEEP				/**< Standard sleep() (<unistd.h>) */
};

#ifdef __cplusplus
}
}
#endif

#endif /* GIT_FUNCTION_INFO_H */
