// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file IndirectRefOperand.h
 * Operand representing a value to which points a pointer represented by
 * another operand.
 */
#ifndef GIT_INDIRECT_REF_OPERAND_H
#define GIT_INDIRECT_REF_OPERAND_H

#include "DecoratingOperand.h"

#include <boost/serialization/serialization.hpp>

namespace giterator { 

/**
 * Operand representing an object pointed to by a pointer (unary `*' in C).
 * The operand is the dereferenced pointer, which is always either pointer
 * or reference type.
 * These operands get created from GIMPLE @c INDIRECT_REF tree nodes.
 *
 * @see create_git_operand_from_tree() in gimplexx-bridge.c to see how indirect reference
 * operands get created from GIMPLE operands
 */
class IndirectRefOperand : public detail::DecoratingOperand {
  public:
	/**
	 * Constructor, creates a new indirect reference operand.
	 * Empty constructor exists for serialization purposes,
	 * use nonempty for normal use.
	 */
	IndirectRefOperand () {}
    /**
     * Constructor, creates a new indirect reference operand.
     * 
     * @param type the type of the new operand
     * @param oper the operand representing the pointer to value, 
     *      that will the new operand represent
     */
    IndirectRefOperand (const Type* type, std::auto_ptr<Operand> oper) : 
        detail::DecoratingOperand (type, oper) {}
    
    virtual void accept (Visitor& v) const
    { v.visit (*this); }

    friend class boost::serialization::access;
	template<class Archive>
	void serialize(Archive & ar, const unsigned int)
	{
		boost::serialization::void_cast_register(static_cast<IndirectRefOperand *>(NULL), static_cast<Operand *>(NULL));
		ar & boost::serialization::base_object<DecoratingOperand>(*this);
	}
};

}

#endif /* GIT_INDIRECT_REF_OPERAND_H */

