// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file LabelDecl.h
 * Class representing declaration of a label.
 */

#ifndef GIT_LABEL_DECL_H
#define GIT_LABEL_DECL_H

#include "AbstractDecl.h"
#include "Type.h"

#include <boost/serialization/serialization.hpp>

namespace giterator {

/**
 * Class representing declaration of a label - destination of
 * an unconditional jump (goto statement).
 * Labels appear in function bodies in block scopes.
 * Label declarations have no data type, therefore NULL is to be passed
 * as a type argument in constructor and @c getType() for this declaration
 * should never be used.
 *
 * @see create_git_decl_from_tree in gimplexx-bridge.c to see how declarations of
 * labels get created from GIMPLE declarations
 */
class LabelDecl : public detail::AbstractDecl {
  public:
	/**
	 * Constructor, creates a new label declaration.
	 * Empty constructor exists for serialization purposes,
	 * use nonempty for normal use.
	 */
	LabelDecl () : detail::AbstractDecl() {}

	/**
	 * Constructor, creates a new label declaration.
	 *
     * @param uid unique ID of the new label declaration
     * @param name name of the new label declaration
     * @param type type of the new label declaration (NULL)
     */
    LabelDecl (unsigned int uid, const std::string& name, const Type* type) :
                detail::AbstractDecl (uid, name, type) {}

    void accept (Visitor& visitor) const
    { visitor.visit (*this); }

    friend class boost::serialization::access;
	template<class Archive>
	void save(Archive & ar, const unsigned int) const
	{
		ar << boost::serialization::base_object<AbstractDecl>(*this);
	}

	template<class Archive>
	void load(Archive & ar, const unsigned int)
	{
		ar >> boost::serialization::base_object<AbstractDecl>(*this);
	}

	template<class Archive>
	void serialize(Archive & ar, const unsigned int version)
	{
		boost::serialization::void_cast_register(static_cast<LabelDecl *>(NULL), static_cast<TreeProduct *>(NULL));
		boost::serialization::void_cast_register(static_cast<LabelDecl *>(NULL), static_cast<Decl *>(NULL));
		boost::serialization::split_member(ar, *this, version);
	}
};

}

#endif /* GIT_LABEL_DECL_H */
