// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file NopStmt.h
 * Class representing GIMPLE_NOP statement.
 */
#ifndef GIT_NOP_STATEMENT_H
#define GIT_NOP_STATEMENT_H

#include "Stmt.h"

#include <boost/serialization/serialization.hpp>

namespace giterator {

/**
 * Class representing the @c GIMPLE_NOP statement - a 'do nothing' statement.
 * This statement does not do anything and has no operands, therefore
 * these statements could be ignored during processing of GIMPLE
 * representation of a program.
 *
 * @see gimple_nop_to_git_stmt() in gimplexx-bridge.c to see how
 * a no operation statement gets created from GIMPLE tree node.
 */
class NopStmt : public Stmt {
  public:
	/**
	 * Constructor, creates a new no operation statement.
	 */
	NopStmt() {}

    void accept (Stmt::Visitor& visitor) const
    { visitor.visit (*this); }

    friend class boost::serialization::access;
	template<class Archive>
	void serialize(Archive &, const unsigned int)
	{
		boost::serialization::void_cast_register(static_cast<NopStmt *>(NULL), static_cast<Stmt *>(NULL));
	}
};

}

#endif /* #define GIT_NOP_STATEMENT_H */

