// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file OffsetDecl.h
 * Base class for representing declarations with offset mapper.
 */
#ifndef GIT_OFFSET_DECL_H
#define GIT_OFFSET_DECL_H

#include "AbstractDecl.h"
#include "OffsetMapper.h"

#include <boost/serialization/serialization.hpp>

namespace giterator {

/**
 * Base class for representing declarations with offset mapper.
 *
 * @see giterator::OffsetMapper for more detail about purpose
 * and usage of offset mapper.
 */
class OffsetDecl : public detail::AbstractDecl {
private:
	/** Mapper specifying layout of this declaration
	 * @see giterator::OffsetMapper
	 */
	OffsetMapper m_fieldsMapper;
  public:
	/**
	 * Constructor, creates a new offset declaration.
	 * Empty constructor exists for serialization purposes,
	 * use nonempty for normal use.
	 */
	OffsetDecl () : detail::AbstractDecl(), m_fieldsMapper(NULL) {}

	/**
	 * Constructor, creates a new offset declaration.
	 *
	 * @param uid unique ID of the new offset declaration
     * @param name name of the new offset declaration
     * @param type type of the new offset declaration,
     * that is used to initialize the offset mapper
     * @see m_fieldsMapper
	 */
	OffsetDecl (unsigned int uid, const std::string& name, const Type* type) :
        detail::AbstractDecl (uid, name, type), m_fieldsMapper(type) {}

	/**
	 * Obtain the offset mapper determining layout of this declaration.
	 *
	 * @return offset mapper determining layout of this declaration
	 */
    const OffsetMapper* getFieldsMapper () const
	{ return &m_fieldsMapper; }

    friend class boost::serialization::access;
	template<class Archive>
	void save(Archive & ar, const unsigned int) const
	{
		ar << boost::serialization::base_object<AbstractDecl>(*this);
	}

	template<class Archive>
	void load(Archive & ar, const unsigned int)
	{
		ar >> boost::serialization::base_object<AbstractDecl>(*this);

		if (typeid(*getType()) == typeid (const RecordType)
				|| typeid(*getType()) == typeid (const PtrType)
				|| typeid(*getType()) == typeid (const ArrayType))
			m_fieldsMapper.create(getType());
	}

	template<class Archive>
	void serialize(Archive & ar, const unsigned int version)
	{
		boost::serialization::split_member(ar, *this, version);
	}
};

}

#endif /* GIT_OFFSET_DECL_H */
