// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file OffsetMapper.cpp
 * Definitions for class for classes mapping variables to offsets.
 */

#include "OffsetMapper.h"

namespace giterator
{

OffsetMapper::OffsetMapper (const Type* type): m_created(false), m_size (0)
{
	if (type)
		create(type);
}

void OffsetMapper::create(const Type* type)
{
	// TODO
	const RecordType* recType = dynamic_cast<const RecordType*> (type);
	const PtrType* ptrType = dynamic_cast<const PtrType*> (type);
	const ArrayType* arrType = dynamic_cast<const ArrayType*> (type);
	if (ptrType)
	{
		recType = dynamic_cast<const RecordType*> (ptrType->getPointingToType ());
	}
	else if (arrType)
	{
		recType = dynamic_cast<const RecordType*> (arrType->getElementType ());
	}

	if (!recType)
	{
		m_created = false;
		return;
	}
	else
		m_created = true;

	std::pair<RecordType::FieldsIterator, RecordType::FieldsIterator> fields = recType->fields ();
	for (RecordType::FieldsIterator i1 = fields.first; i1 != fields.second; ++i1)
	{
		addVariable(i1->second);
	}
}

size_t OffsetMapper::getOffset (unsigned int uid) const
{
	if (!m_created || !uid)
		return 0;

	Offsets::const_iterator it = m_offsets.find (uid);
	assert (it != m_offsets.end ());

	return it->second;
}

size_t OffsetMapper::getSize () const
{ return m_size; }

OffsetMapper::OffsetMapper () : m_created(false), m_size (0)  { }

void OffsetMapper::addVariable (const giterator::Decl* decl)
{
    assert (m_offsets.find (decl->getUid ()) == m_offsets.end ());

	m_offsets.insert (std::make_pair (decl->getUid (), m_size));
	m_size += decl->getType ()->getUnitSize ();
}

}
