// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file ParamDecl.h
 * Class representing declaration of parameter of a function.
 */
#ifndef GIT_PARAM_DECL_H
#define GIT_PARAM_DECL_H

#include "OffsetDecl.h"

#include <boost/serialization/serialization.hpp>

namespace giterator {
    
/**
 * Class representing declaration of parameter of a function.
 * In GIMPLE++, they are equivalent to VarDecls, except they
 * are accessible through @c Function class.
 * Parameter declarations get created from GIMPLE @c PARAM_DECL
 * nodes.
 *
 * @see create_git_decl_from_tree() in gimplexx-bridge.c
 * to see how declarations of function parameters
 * get created from GIMPLE declarations
 *
 * @see giterator::Function
 */
class ParamDecl : public OffsetDecl {
  public:
	/**
	 * Constructor, creates a new function parameter declaration.
	 * Empty constructor exists for serialization purposes,
	 * use nonempty for normal use.
	 */
	ParamDecl () : OffsetDecl() {}
	/**
	 * Constructor, creates a new function parameter declaration.
	 *
     * @param uid unique ID of the new declaration
     * @param name name of the new declaration
     * @param type type of the new declaration
     */
    ParamDecl (unsigned int uid, const std::string& name, const Type* type) :
        OffsetDecl (uid, name, type) {}
    
    void accept (Visitor& visitor) const
    { visitor.visit (*this); }

    friend class boost::serialization::access;
	template<class Archive>
	void serialize(Archive & ar, const unsigned int version)
	{
		boost::serialization::void_cast_register(static_cast<ParamDecl *>(NULL), static_cast<TreeProduct *>(NULL));
		boost::serialization::void_cast_register(static_cast<ParamDecl *>(NULL), static_cast<Decl *>(NULL));
		ar & boost::serialization::base_object<OffsetDecl>(*this);
	}
};

}

#endif /* GIT_PARAM_DECL_H */
