// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file ResultDecl.h
 * Class representing declaration of a result(to return from function).
 */
#ifndef GIT_RESULT_DECL_H
#define GIT_RESULT_DECL_H

#include "AbstractDecl.h"
#include "Type.h"

#include <boost/serialization/serialization.hpp>

namespace giterator {

/**
 * Class representing declaration of a result
 * (to return from function).
 * Result declarations get created from GIMPLE @c RESULT_DECL
 * nodes. They are equivalent to @c VarDecl.
 *
 * @see create_git_decl_from_tree() in gimplexx-bridge.c
 * to see how declarations of results get created from GIMPLE declarations
 *
 * @see VarDecl
 */
class ResultDecl : public detail::AbstractDecl {
  public:
	/**
	 * Constructor, creates a new result declaration.
	 * Empty constructor exists for serialization purposes,
	 * use nonempty for normal use.
	 */
	ResultDecl () : detail::AbstractDecl() {}
	/**
	 * Constructor, creates a new result declaration.
	 *
     * @param uid unique ID of the new declaration
     * @param name name of the new declaration
     * @param type type of the new declaration
     */
    ResultDecl (unsigned int uid, const std::string& name, const Type* type) :
                detail::AbstractDecl (uid, name, type) {}

    void accept (Visitor& visitor) const
    { visitor.visit (*this); }

    friend class boost::serialization::access;
	template<class Archive>
	void save(Archive & ar, const unsigned int) const
	{
		ar << boost::serialization::base_object<AbstractDecl>(*this);
	}

	template<class Archive>
	void load(Archive & ar, const unsigned int)
	{
		ar >> boost::serialization::base_object<AbstractDecl>(*this);
	}

	template<class Archive>
	void serialize(Archive & ar, const unsigned int version)
	{
		boost::serialization::void_cast_register(static_cast<ResultDecl *>(NULL), static_cast<TreeProduct *>(NULL));
		boost::serialization::void_cast_register(static_cast<ResultDecl *>(NULL), static_cast<Decl *>(NULL));
		boost::serialization::split_member(ar, *this, version);
	}
};

}

#endif /* GIT_RESULT_DECL_H */
