// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file SerializationExport.h
 * Header containing one macro  for each serialized class,
 * defining unique id of a class needed for boost::serialization.
 */
#ifndef GIT_SERIALIZATION_EXPORT_H
#define GIT_SERIALIZATION_EXPORT_H

#include <boost/serialization/export.hpp>
#include <boost/serialization/extended_type_info.hpp>

#include "Types.h"
#include "Decls.h"
#include "Constants.h"
#include "Stmts.h"
#include "Operations.h"
#include "Operands.h"
#include "Gimple.h"

#include "TreeProduct.h"
#include "Function.h"

BOOST_CLASS_EXPORT_GUID(giterator::detail::TreeProduct, "TreeProduct")
BOOST_CLASS_EXPORT_GUID(giterator::Gimple, "Gimple")

BOOST_CLASS_EXPORT_GUID(giterator::Function, "Function")
BOOST_CLASS_EXPORT_GUID(giterator::Block, "Block")
BOOST_CLASS_EXPORT_GUID(giterator::Edge, "Edge")

BOOST_CLASS_EXPORT_GUID(giterator::Type, "Type")
BOOST_CLASS_EXPORT_GUID(giterator::detail::AbstractType, "AbstractType")
BOOST_CLASS_EXPORT_GUID(giterator::ArrayType, "ArrayType")
BOOST_CLASS_EXPORT_GUID(giterator::BoolType, "BoolType")
BOOST_CLASS_EXPORT_GUID(giterator::EnumType, "EnumType")
BOOST_CLASS_EXPORT_GUID(giterator::FunctionType, "FunctionType")
BOOST_CLASS_EXPORT_GUID(giterator::IntegerType, "IntegerType")
BOOST_CLASS_EXPORT_GUID(giterator::PtrType, "PtrType")
BOOST_CLASS_EXPORT_GUID(giterator::RealType, "RealType")
BOOST_CLASS_EXPORT_GUID(giterator::RecordType, "RecordType")
BOOST_CLASS_EXPORT_GUID(giterator::VoidType, "VoidType")

BOOST_CLASS_EXPORT_GUID(giterator::Decl, "Decl")
BOOST_CLASS_EXPORT_GUID(giterator::detail::AbstractDecl, "AbstractDecl")
BOOST_CLASS_EXPORT_GUID(giterator::OffsetDecl, "OffsetDecl")
BOOST_CLASS_EXPORT_GUID(giterator::ConstDecl, "ConstDecl")
BOOST_CLASS_EXPORT_GUID(giterator::FieldDecl, "FieldDecl")
BOOST_CLASS_EXPORT_GUID(giterator::FunctionDecl, "FunctionDecl")
BOOST_CLASS_EXPORT_GUID(giterator::LabelDecl, "LabelDecl")
BOOST_CLASS_EXPORT_GUID(giterator::ParamDecl, "ParamDecl")
BOOST_CLASS_EXPORT_GUID(giterator::ResultDecl, "ResultDecl")
BOOST_CLASS_EXPORT_GUID(giterator::TypeDecl, "TypeDecl")
BOOST_CLASS_EXPORT_GUID(giterator::VarDecl, "VarDecl")
BOOST_CLASS_EXPORT_GUID(giterator::RecordDecl, "RecordDecl")

BOOST_CLASS_EXPORT_GUID(giterator::Constant, "Constant")
BOOST_CLASS_EXPORT_GUID(giterator::IntegerConstant, "IntegerConstant")
BOOST_CLASS_EXPORT_GUID(giterator::EnumeralConstant, "EnumeralConstant")
BOOST_CLASS_EXPORT_GUID(giterator::RealConstant, "RealConstant")
BOOST_CLASS_EXPORT_GUID(giterator::StringConstant, "StringConstant")

BOOST_CLASS_EXPORT_GUID(giterator::Stmt, "Stmt")
BOOST_CLASS_EXPORT_GUID(giterator::CallStmt, "CallStmt")
BOOST_CLASS_EXPORT_GUID(giterator::ReturnStmt, "ReturnStmt")
BOOST_CLASS_EXPORT_GUID(giterator::AssignStmt, "AssignStmt")
BOOST_CLASS_EXPORT_GUID(giterator::CondStmt, "CondStmt")
BOOST_CLASS_EXPORT_GUID(giterator::LabelStmt, "LabelStmt")
BOOST_CLASS_EXPORT_GUID(giterator::GotoStmt, "GotoStmt")
BOOST_CLASS_EXPORT_GUID(giterator::NopStmt, "NopStmt")
BOOST_CLASS_EXPORT_GUID(giterator::PhiStmt, "PhiStmt")
BOOST_CLASS_EXPORT_GUID(giterator::SwitchStmt, "SwitchStmt")

BOOST_CLASS_EXPORT_GUID(giterator::Operand, "Operand")
BOOST_CLASS_EXPORT_GUID(giterator::ArrayRefOperand, "ArrayRefOperand")
BOOST_CLASS_EXPORT_GUID(giterator::ComponentRefOperand, "ComponentRefOperand")
BOOST_CLASS_EXPORT_GUID(giterator::LabelExprOperand, "LabelExprOperand")
BOOST_CLASS_EXPORT_GUID(giterator::CstOperand, "CstOperand")
BOOST_CLASS_EXPORT_GUID(giterator::DeclOperand, "DeclOperand")
BOOST_CLASS_EXPORT_GUID(giterator::detail::DecoratingOperand, "DecoratingOperand")
BOOST_CLASS_EXPORT_GUID(giterator::detail::TypeOperand, "TypeOperand")
BOOST_CLASS_EXPORT_GUID(giterator::ConstructorOperand, "ConstructorOperand")
BOOST_CLASS_EXPORT_GUID(giterator::AddrOperand, "AddrOperand")
BOOST_CLASS_EXPORT_GUID(giterator::IndirectRefOperand, "IndirectRefOperand")
BOOST_CLASS_EXPORT_GUID(giterator::ArrayConstructorOperand, "ArrayConstructorOperand")
BOOST_CLASS_EXPORT_GUID(giterator::RecordConstructorOperand, "RecordConstructorOperand")

BOOST_CLASS_EXPORT_GUID(giterator::Operation, "Operation")
BOOST_CLASS_EXPORT_GUID(giterator::BinaryOperation, "BinaryOperation")
BOOST_CLASS_EXPORT_GUID(giterator::detail::AbstractOperation, "AbstractOperation")
BOOST_CLASS_EXPORT_GUID(giterator::detail::AbstractBinaryOperation, "AbstractBinaryOperation")
BOOST_CLASS_EXPORT_GUID(giterator::detail::AbstractUnaryOperation, "AbstractUnaryOperation")
BOOST_CLASS_EXPORT_GUID(giterator::UnaryOperation, "UnaryOperation")
BOOST_CLASS_EXPORT_GUID(giterator::detail::UnaryOperationImpl, "UnaryOperationImpl")
BOOST_CLASS_EXPORT_GUID(giterator::detail::BinaryOperationImpl, "BinaryOperationImpl")

BOOST_CLASS_EXPORT_GUID(giterator::NegateOperation, "NegateOperation")
BOOST_CLASS_EXPORT_GUID(giterator::AbsOperation, "AbsOperation")
BOOST_CLASS_EXPORT_GUID(giterator::FixTruncOperation, "FixTruncOperation")
BOOST_CLASS_EXPORT_GUID(giterator::FloatOperation, "FloatOperation")
BOOST_CLASS_EXPORT_GUID(giterator::BitNotOperation, "BitNotOperation")
BOOST_CLASS_EXPORT_GUID(giterator::TruthNotOperation, "TruthNotOperation")
BOOST_CLASS_EXPORT_GUID(giterator::ParenOperation, "ParenOperation")
BOOST_CLASS_EXPORT_GUID(giterator::NonLvalueOperation, "NonLvalueOperation")
BOOST_CLASS_EXPORT_GUID(giterator::NoOperation, "NoOperation")
BOOST_CLASS_EXPORT_GUID(giterator::ConvertOperation, "ConvertOperation")

BOOST_CLASS_EXPORT_GUID(giterator::PlusOperation, "PlusOperation")
BOOST_CLASS_EXPORT_GUID(giterator::MinusOperation, "MinusOperation")
BOOST_CLASS_EXPORT_GUID(giterator::MultOperation, "MultOperation")
BOOST_CLASS_EXPORT_GUID(giterator::RdivOperation, "RdivOperation")
BOOST_CLASS_EXPORT_GUID(giterator::TruncDivOperation, "TruncDivOperation")
BOOST_CLASS_EXPORT_GUID(giterator::FloorDivOperation, "FloorDivOperation")
BOOST_CLASS_EXPORT_GUID(giterator::CeilDivOperation, "CeilDivOperation")
BOOST_CLASS_EXPORT_GUID(giterator::RoundDivOperation, "RoundDivOperation")
BOOST_CLASS_EXPORT_GUID(giterator::TruncModOperation, "TruncModOperation")
BOOST_CLASS_EXPORT_GUID(giterator::FloorModOperation, "FloorModOperation")
BOOST_CLASS_EXPORT_GUID(giterator::CeilModOperation, "CeilModOperation")
BOOST_CLASS_EXPORT_GUID(giterator::RoundModOperation, "RoundModOperation")
BOOST_CLASS_EXPORT_GUID(giterator::ExactDivOperation, "ExactDivOperation")
BOOST_CLASS_EXPORT_GUID(giterator::MinOperation, "MinOperation")
BOOST_CLASS_EXPORT_GUID(giterator::MaxOperation, "MaxOperation")
BOOST_CLASS_EXPORT_GUID(giterator::LshiftOperation, "LshiftOperation")
BOOST_CLASS_EXPORT_GUID(giterator::RshiftOperation, "RshiftOperation")
BOOST_CLASS_EXPORT_GUID(giterator::LrotateOperation, "LrotateOperation")
BOOST_CLASS_EXPORT_GUID(giterator::RrotateOperation, "RrotateOperation")
BOOST_CLASS_EXPORT_GUID(giterator::BitIorOperation, "BitIorOperation")
BOOST_CLASS_EXPORT_GUID(giterator::BitXorOperation, "BitXorOperation")
BOOST_CLASS_EXPORT_GUID(giterator::BitAndOperation, "BitAndOperation")
BOOST_CLASS_EXPORT_GUID(giterator::TruthAndOperation, "TruthAndOperation")
BOOST_CLASS_EXPORT_GUID(giterator::TruthOrOperation, "TruthOrOperation")
BOOST_CLASS_EXPORT_GUID(giterator::TruthXorOperation, "TruthXorOperation")
BOOST_CLASS_EXPORT_GUID(giterator::LtOperation, "LtOperation")
BOOST_CLASS_EXPORT_GUID(giterator::LeOperation, "LeOperation")
BOOST_CLASS_EXPORT_GUID(giterator::GtOperation, "GtOperation")
BOOST_CLASS_EXPORT_GUID(giterator::GeOperation, "GeOperation")
BOOST_CLASS_EXPORT_GUID(giterator::EqOperation, "EqOperation")
BOOST_CLASS_EXPORT_GUID(giterator::NeOperation, "NeOperation")
BOOST_CLASS_EXPORT_GUID(giterator::OrderedOperation, "OrderedOperation")
BOOST_CLASS_EXPORT_GUID(giterator::UnorderedOperation, "UnorderedOperation")
BOOST_CLASS_EXPORT_GUID(giterator::UnLtOperation, "createUnLtOperation")
BOOST_CLASS_EXPORT_GUID(giterator::UnLeOperation, "UnLeOperation")
BOOST_CLASS_EXPORT_GUID(giterator::UnGtOperation, "UnGtOperation")
BOOST_CLASS_EXPORT_GUID(giterator::UnGeOperation, "UnGeOperation")
BOOST_CLASS_EXPORT_GUID(giterator::UnEqOperation, "UnEqOperation")
BOOST_CLASS_EXPORT_GUID(giterator::LtGtOperation, "LtGtOperation")
BOOST_CLASS_EXPORT_GUID(giterator::WidenSumOperation, "WidenSumOperation")
BOOST_CLASS_EXPORT_GUID(giterator::WidenMultOperation, "WidenMultOperation")

#endif /* GIT_SERIALIZATION_EXPORT_H */
