// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file Serializer.cpp
 * Definitions of functions usable from C code that start
 * serialization and deserialization of GIMPLE++.
 */
#include "Serializer.h"

#include "SerializationExport.h"
#include "Gimple.h"

#include <iostream>
#include <fstream>
#include <boost/archive/text_oarchive.hpp>
#include <boost/archive/text_iarchive.hpp>

/**
 * This namespace contains the implementation of the C++ part of GIMPLE Interpreter.
 */
namespace giterator {
    
/**
 * Function that returns the reference to the structure containing the GIMPLE++ instance.
 *
 * @return the GIMPLE++ instance
 */
Gimple&
getGimpleInstance ()
{
    static std::auto_ptr<Gimple> gimple_;
    
    if (!gimple_.get ())
        gimple_.reset (new Gimple ());
    
    return *gimple_.get ();
}

int serialize(Gimple& gimple, const char* filename)
{
	// create and open a character archive for output
	std::ofstream ofs(filename);
	// save data to archive
	boost::archive::text_oarchive oa(ofs);

	try
	{
		// write class instance to archive
		oa << gimple;
	}
	catch(boost::archive::archive_exception& e)
	{
		std::cerr << "Gimple serialization failed. " << e.what() << std::endl;
		return -1;
	}

	return 0;
}

int deserialize (Gimple& g, const char* filename)
{
	try
	{
		// create and open an archive for input
		std::ifstream ifs(filename);
		boost::archive::text_iarchive ia(ifs);

		std::cerr << "Deserializing...";
		// read class state from archive
		ia >> g;
	}
	catch(boost::archive::archive_exception& e)
	{
		std::cerr << "Gimple deserialization failed." << e.what() << std::endl;
		return -1;
	}

	g.deserialize_routine();

	std::cerr << "OK." << std::endl;
        
	return 0;
}

/**
 * Serialize the gimple representation of the program under analysis.
 */
void
gimplexx_serialize (Gimple* gimple, const char* filename)
{
	std::cerr << "Serializing...";
	if (serialize(*gimple, filename) == 0)
		std::cerr <<  "OK" << std::endl;
	else
		std::cerr <<  "failed" << std::endl;

	return;
}

}
