// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file Serializer.h
 * Declarations of functions usable from C code that start
 * serialization and deserialization of GIMPLE++.
 */
#ifndef GIT_SERIALIZER_H
#define GIT_SERIALIZER_H

#include "CTypes.h"

#ifdef __cplusplus
extern "C" {
namespace giterator {

/**
 * Function that serializes GIMPLE++ representation of a program.
 * Serialization is a process that flattens a data structure into
 * a sequence of characters or bits and stores them in a file
 * that can be later turned back to the original object.
 *
 * In GMC, after processing the whole GIMPLE input and creating
 * a GIMPLE++ representation, serialization routine is executed
 * and the whole GIMPLE++ representation gets dumped to a file.
 * Then, when running the Interpreter part, all dumped data
 * structures get retrieved (deserialized) and Interpreter can start.
 * This allows to separate the Iterator and Interpreter parts
 * and run the Interpreter independently.
 *
 * @param gimple the GIMPLE++ representation of a program to serialize
 * @param filename name of the file to use for serialization
 * @return 0 if successful, -1 otherwise
 *
 * @see int deserialize(giterator::Gimple& gimple, const char* filename);
 */
int serialize(giterator::Gimple& gimple, const char* filename);

/**
 * Function that deserializes GIMPLE++ representation of a program
 * from a file that was created by previous serialization.
 *
 * @param gimple holder for the GIMPLE++ representation of a program
 * where to store the deserialized representation
 * @param filename name of the file which to deserialize
 * @return 0 if successful, -1 otherwise
 *
 * @see int serialize(giterator::Gimple& gimple, const char* filename);
 */
int deserialize(giterator::Gimple& gimple, const char* filename);

#endif

extern void gimplexx_serialize (git_gimple gimple, const char* filename);

#ifdef __cplusplus
}}
#endif
    
#endif /* GIT_SERIALIZER_H */

