// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file TypeOperand.h
 * Base class for all operands that store its type.
 */
#ifndef GIT_TYPE_OPERAND_H
#define GIT_TYPE_OPERAND_H

#include "Operand.h"

#include <boost/serialization/serialization.hpp>
#include <memory>


namespace giterator { namespace detail {

/**
 * Base class for all operands that store its type.
 * These are for example array constructor operands,
 * which have to store the type of the array they construct.
 */
class TypeOperand : public Operand {
    private:
        /*! Type associated with this operand. */
        const Type* m_type;

    public:
        /**
         * Constructor, creates a new type operand.
         * Empty constructor exists for serialization purposes,
         * use nonempty for normal use.
         */
        TypeOperand () {}
        /**
         * Constructor, creates a new type operand.
         *
         * @param type type of this operand
         */
        TypeOperand (const Type* type) :
            m_type (type) {}

        /**
         * Returns the type of this operand.
         *
         * @return the type of this operand
         */
        const Type* getType () const
        { return m_type; }

		friend class boost::serialization::access;
		template<class Archive>
		void save(Archive & ar, const unsigned int) const
		{
			ar << m_type;
		}

		template<class Archive>
		void load(Archive & ar, const unsigned int)
		{
			Type* type;
			ar >> type;
			m_type = (const Type*) type;
		}

		template<class Archive>
		void serialize(Archive & ar, const unsigned int version)
		{
			boost::serialization::split_member(ar, *this, version);
		}
};

}}

#endif /* GIT_TYPE_OPERAND_H */
