// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file VoidType.h
 * Class representing a void type.
 */
#ifndef GIT_VOID_TYPE_H
#define GIT_VOID_TYPE_H

#include "AbstractType.h"

#include <typeinfo>
#include <boost/serialization/serialization.hpp>

namespace giterator { 
    
/**
 * Class representing a void type.
 * Void types get created from GIMPLE @c VOID_TYPE nodes.
 *
 * @see create_git_type_from_tree() in gimplexx-bridge.c
 * to see how void types get created from GIMPLE types
 */
class VoidType : public detail::AbstractType {
  public:

	/**
	 * Constructor, creates a new void type.
	 *
	 * Empty constructor exists for serialization purposes,
	 * use nonempty for normal use.
	 */
	VoidType() {}

    /**
     * Constructor, creates a new void type.
     */
    VoidType(const std::string& name) : detail::AbstractType (name, 0) {}

    /**
     * Type comparison.
     * Compare this type to another types.
     * Two void types are always the same (flase is returned
     * only if compared type is not void).
     *
     * @param type the type this type is compared to
     * @return true if the types are the same, false otherwise
     */
    bool isSame (const Type& type) const
    {
      return typeid (VoidType) == typeid (type);
    }

    friend class boost::serialization::access;
	template<class Archive>
	void serialize(Archive & ar, const unsigned int)
	{
		boost::serialization::void_cast_register(static_cast<VoidType *>(NULL), static_cast<TreeProduct *>(NULL));
		boost::serialization::void_cast_register(static_cast<VoidType *>(NULL), static_cast<Type *>(NULL));
		ar & boost::serialization::base_object<AbstractType>(*this);
	}
};

}

#endif  /* GIT_VOID_TYPE_H */

