// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file AbstractVarsMapper.cpp
 * Definitions for base class for classes mapping variables to offsets.
 */
#include "AbstractVarsMapper.h"

#include "../git/Decl.h"
#include "../git/VarDecl.h"
#include "../git/Type.h"

#include "../git/RecordType.h"
#include "../git/PtrType.h"

#include "../Error.h"

#include <typeinfo>

using namespace giterator;


namespace ginterpreter {
    
/**
 * Adds a variable into this mapping.
 * The size is incremented by the size of the variable type.
 * 
 * @param decl declaration of the variable
 */
void
AbstractVarsMapper::addVariable (const Decl* decl)
{
    assert (m_offsets.find (decl->getUid ()) == m_offsets.end ());

	m_offsets.insert (std::make_pair (decl->getUid (), m_size));
	m_size += decl->getType ()->getUnitSize ();
}


/**
 * Returns the offset for the specified variable.
 * 
 * @param uid the unique ID of the declaration of the variable
 * @return offset of the variable
 */
size_t
AbstractVarsMapper::getVarOffset (unsigned int uid) const
{
    Offsets::const_iterator it = m_offsets.find (uid);
    assert (it != m_offsets.end ());
    
    return it->second;
}

}
