// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file AbstractVarsMapper.h
 * Declaration of base class for classes mapping variables to offsets.
 */
#ifndef INT_ABSTRACT_VARS_MAPPER_H
#define INT_ABSTRACT_VARS_MAPPER_H

#include "../git/git_fwd.h"

#include <map>

namespace ginterpreter {
    
/**
 * Base class for classes mapping variables to offsets.
 */
class AbstractVarsMapper {
  protected:
    /*! type of the map holding the mapping represented by this class */
    typedef std::map<unsigned int, size_t> Offsets;
    
  private:
    /*! map holding the mapping */
    Offsets m_offsets;
    /*! size used by this mapping */
    size_t m_size;
    
  public:    
    size_t getVarOffset (unsigned int uid) const;
    
    /**
     * Returns the size of this mapping.
     */
    size_t getSize () const
    { return m_size; }
    
  protected:
    AbstractVarsMapper () : m_size (0) { }

    void addVariable (const giterator::Decl* decl);
};

}

#endif /* INT_ABSTRACT_VARS_MAPPER_H */
