// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file AddrResolver.cpp
 * Definitions for class finding addresses of values
 * represented by operand.
 */
#include "AddrResolver.h"

#include "ValueReader.h"
#include "IntegerValue.h"
#include "PtrValue.h"
#include "FunctionValue.h"
#include "RecordValue.h"
#include "ConstantsManager.h"
#include "GlobalsManager.h"

#include "../mm/MemoryState.h"

#include "../git/Operands.h"
#include "../git/Decls.h"

using namespace mmodule;
using namespace giterator;


namespace ginterpreter {
unsigned int m_fieldUid = 0;
unsigned int m_additionalOffset = 0;
size_t m_offset = 0;
size_t m_offset2 = 0;

AddrResolver::AddrResolver (const FunctionValue* func, const MemoryState* ms, 
                            const ConstantsManager* cstManager, const GlobalsManager* globalsManager,
                            Ptr frame)
    : m_func (func), m_memoryState (ms),
      m_cstManager (cstManager), m_globalsManager(globalsManager),
      m_frame (frame),
      m_type (0),
      m_ptr (),
      m_declOffset(0),
      m_decl(NULL)
{
}


void
AddrResolver::visit(const AddrOperand& op)
{
	//ValueReader addrReader (m_func, m_memoryState, m_cstManager, m_globalsManager, m_frame);
	//op.getSubOperand ()->accept (addrReader);

	op.getSubOperand ()->accept(*this);

	const PtrValue* ptrVal = dynamic_cast<const PtrValue*> (m_memoryState->load(m_ptr));
	assert (ptrVal);
	m_ptr = ptrVal->getPtr ();
	m_type = op.getType ();
}


void
AddrResolver::visit (const ParamDecl& decl)
{
    getLocalPtr_ (decl);
    m_decl = &decl;
}
        
void
AddrResolver::visit (const VarDecl& decl)
{
	if (decl.getLocal ())
		getLocalPtr_ (decl);
	else
		getGlobalPtr_ (decl);
	m_decl = &decl;
	/*
	if (typeid(*decl.getType()) == typeid(const ArrayType))
	{
		LOG_DEBUG("arr %s", decl.getName().c_str());
		giterator::OffsetMapper om(decl.getType());
		decl.getFieldsMapper()->getOffset(22);
	}*/
}

void
AddrResolver::visit (const RecordDecl& decl)
{
	if (decl.getLocal ())
		getLocalPtr_ (decl);
	else
		getGlobalPtr_ (decl);

	m_decl = &decl;
}
        
        
void 
AddrResolver::visit (const FunctionDecl& decl)
{
    m_ptr = m_cstManager->getFuncPtr (&decl);
}

void
AddrResolver::getLocalPtr_ (const giterator::Decl& decl)
{
	m_declOffset = m_func->getVarsMapper ()->getVarOffset (decl.getUid ());
    m_ptr = m_frame + m_declOffset;
}

void
AddrResolver::getGlobalPtr_ (const giterator::Decl& decl)
{
	m_declOffset = m_globalsManager->getGlobalDeclOffset(&decl);
	m_ptr = m_globalsManager->getGlobalDeclPtr (&decl);
}

void AddrResolver::visit (const FieldDecl& decl)
{
	assert(m_decl);
	m_ptr += m_decl->getFieldsMapper()->getOffset(decl.getUid());
	m_decl = &decl;
}

void AddrResolver::visit (const LabelDecl& decl)
{
	getLocalPtr_ (decl);
}

void AddrResolver::visit (const ResultDecl& decl)
{
	getLocalPtr_ (decl);
}

void AddrResolver::visit (const TypeDecl& decl)
{
	if (decl.getLocal ())
		getLocalPtr_ (decl);
	else
		getGlobalPtr_ (decl);
}

void AddrResolver::visit (const ConstDecl& decl)
{
	if (decl.getLocal ())
		getLocalPtr_ (decl);
	else
		getGlobalPtr_ (decl);
}

void
AddrResolver::visit(const DeclOperand& op)
{
    op.getDecl()->accept (*this);
    m_type = op.getDecl ()->getType ();
}

void
AddrResolver::visit(const LabelExprOperand& op)
{
	// should not be visited as whole
	assert (false);
}

void AddrResolver::visit (const ArrayConstructorOperand& op)
{
	// should not be visited as whole
	assert (false);
}

void AddrResolver::visit (const RecordConstructorOperand& op)
{
	// should not be visited as whole
	assert (false);
}

void
AddrResolver::visit(const IndirectRefOperand& op)
{
    //ValueReader addrReader (m_func, m_memoryState, m_cstManager, m_globalsManager, m_frame);
    //op.getSubOperand ()->accept (addrReader);
	op.getSubOperand ()->accept (*this);
    
    const PtrValue* ptrVal = dynamic_cast<const PtrValue*> (m_memoryState->load(m_ptr));
    assert (ptrVal);

    m_ptr = ptrVal->getPtr ();
    m_type = op.getType ();
}


void
AddrResolver::visit(const ArrayRefOperand& op)
{
    //AddrResolver addRes (m_func, m_memoryState, m_cstManager, m_globalsManager, m_frame);
    op.getArray ()->accept (*this);
    //Ptr arrayPtr = addRes.getPtr ();
    
    const giterator::ArrayType* arrayType =
            dynamic_cast<const giterator::ArrayType*> (getTargetType ());
    assert (arrayType);
    const giterator::Type* elementType = arrayType->getElementType ();
    size_t elementSize = elementType->getUnitSize ();
    
    ValueReader indexReader (m_func, m_memoryState, m_cstManager, m_globalsManager, m_frame);
    op.getIndex ()->accept (indexReader);
    const IntegerValue* index = dynamic_cast<const IntegerValue*> (indexReader.getValue ());
    assert (index);
    
    //m_ptr = arrayPtr + index->getValue () * elementSize;
    m_ptr += index->getValue () * elementSize;
    m_type = elementType;
}

void
AddrResolver::visit(const ComponentRefOperand& op)
{
	//AddrResolver addRes (m_func, m_memoryState, m_cstManager, m_globalsManager, m_frame);

	op.getRecord ()->accept (*this);
	op.getField ()->accept (*this);

	assert(!m_ptr.isNull());

	//m_ptr = addRes.getPtr ();
	m_type = op.getType ();
}
    

void
AddrResolver::visit(const CstOperand& op)
{
    m_ptr = m_cstManager->getCstPtr (op.getConstant ());
    assert (!m_ptr.isNull ());

    m_type = op.getType ();
}

}
