// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file ArrayValue.h
 * Declaration of class representing simulated value of an array.
 */
#ifndef INT_ARRAY_VALUE_H
#define INT_ARRAY_VALUE_H

#include "AbstractValue.h"

namespace ginterpreter {

/**
 * Class representing an array.
 * This class is a regular simulated value, so it can be stored in the simulated memory.
 * In GMC however, ArrayValue is not stored to the simulated memory.
 * It is only a helper class that is returned from @c ValueReader
 * It holds the pointer to the beginning of an array
 * as well as the size of an array.
 * @c ValueWriter reads all the values overlapping over area <ptr, ptr+size>
 * and stores them one by one to the simulated memory.
 */
class ArrayValue : public AbstractPtrValue {
  private:
	/*! pointer to the beginning of the array */
	mmodule::Ptr m_ptr;
	/*! size of the array */
	size_t m_size;
  public:
    /**
     * Constructor.
     *
     * @param type GIMPLE++ representation of the type of the array
     * @param arrayPtr pointer to the beginning of the array
     * @param size size of the array starting at @c ptr
     */
	ArrayValue(const giterator::Type* type, mmodule::Ptr arrayPtr, size_t size);

	/**
	 * Clone this value.
	 *
	 * @returns an exact copy of this value.
	 */
    Value* clone () const;

    /**
     * Get a pointer to the beginning of the array.
     *
     * @return pointer to the beginning of the array
     */
    mmodule::Ptr getPtr () const;

    /**
     * Get a size of this array in bytes.
     *
     * @return size of this array
     */
    size_t getUnitSize () const;
};

}

#endif /* INT_ARRAY_VALUE_H */
