// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file BoolValue.cpp
 * Definitions for class representing boolean simulated value.
 */
#include "BoolValue.h"
#include "IntegerValue.h"
#include "RealValue.h"

#include <typeinfo>
#include <cassert>

namespace ginterpreter
{

BoolValue::BoolValue (const giterator::Type* type, bool value) :
			AbstractNonPtrValue (type), m_value (value)
{}

bool
BoolValue::getValue () const
{
	return m_value;
}

Value*
BoolValue::clone () const
{
	return new BoolValue (getType (), m_value);
}

mmodule::Hash
BoolValue::getHash () const
{
	return m_value ? 1 : 0;
}

bool
BoolValue::operator== (const Value& v) const
{
	checkType (v);

	if (typeid(v) == typeid(BoolValue))
	{
		const BoolValue& bv = dynamic_cast<const BoolValue&> (v);
		return m_value == bv.getValue();
	}
	else if (typeid(v) == typeid(IntegerValue))
	{
		const IntegerValue& iv = dynamic_cast<const IntegerValue&> (v);
		return m_value == iv.getValue();
	}
	else if (typeid(v) == typeid(RealValue))
	{
		const RealValue& rv = dynamic_cast<const RealValue&> (v);
		return m_value == rv.getValue();
	}
	else
		return false;
}

void
BoolValue::print (std::ostream& os) const
{
	if (m_value)
		os << "true";
	else
		os << "false";
}

void
BoolValue::checkType (const Value& b) const
{
  assert (getType()->isSame(*b.getType()));     (void)b;
}

}
