// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file BoolValue.h
 * Declaration of class representing boolean simulated value.
 */
#ifndef INT_BOOL_VALUE_H
#define INT_BOOL_VALUE_H

#include "AbstractValue.h"

namespace ginterpreter {

class IntegerValue;

/**
 * Class representing boolean simulated value.
 */
class BoolValue : public AbstractNonPtrValue {
  private:
    /*! the value this simulated value represents */
    bool m_value;
    
  public:
    /**
     * Constructor.
     * 
     * @param type type of the simulated value
     * @param value value of the simulated value
     */
    BoolValue (const giterator::Type* type, bool value);
    
    /**
     * Returns the value of this simulated value.
     */
    bool getValue () const;
    
    /**
	 * Clone this value.
	 *
	 * @returns an exact copy of this value.
	 */
    Value* clone () const;
    
    /**
	 * Get a hash of this value.
	 * For memory state equivalency check purposes.
	 *
	 * @return hash of this value.
	 */
    mmodule::Hash getHash () const;
    
    /**
     * Equality operator for the value.
     *
     * @param v value to compare this value to
     * @return true if the values are equal,
     * flase otherwise
     */
    bool operator== (const Value& v) const;

    /**
     * Print the value to a specified stream.
     *
     * @param os the stream where to print the value
     */
    void print (std::ostream& os) const;
  protected:
    /**
	 * Checks if the type of value @c b and the type of this value are the same.
	 *
	 * @param b the value to check if the type is the same
	 *
	 * @see giterator::BoolType::isSame()
	 */
	void checkType (const Value& b) const;
};

}

#endif /* INT_BOOL_VALUE_H */
