// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file BuiltinFunctionValue.cpp
 * Definitions for class representing simulated builtin function.
 */
#include "BuiltinFunctionValue.h"

#include "ConstantsManager.h"
#include "GlobalsManager.h"

#include "../git/FunctionType.h"

#include <memory>
#include <boost/functional/hash.hpp>

namespace ginterpreter
{

BuiltinFunctionValue::BuiltinFunctionValue (enum giterator::function_code code,
											const giterator::FunctionType* type,
											FuncMethod method)
: AbstractNonPtrValue (type), m_code (code), m_method (method)
{}

Value* BuiltinFunctionValue::clone() const
{
	abort ();
}

mmodule::Hash BuiltinFunctionValue::getHash () const
{
	return boost::hash<enum giterator::function_code> () (m_code);
}

enum giterator::function_code BuiltinFunctionValue::getCode () const
{
	return m_code;
}

const giterator::FunctionType* BuiltinFunctionValue::getFunctionType () const
{
	return static_cast<const giterator::FunctionType*> (getType ());
}


BuiltinFunctionValue::FuncMethod BuiltinFunctionValue::getMethod () const
{
	return m_method;
}

}
