// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file ConditionInfo.h
 * Structure specifying the layout of the values in an area
 * holding information about a condition.
 */
#ifndef INT_CONDITION_INFO_H
#define INT_CONDITION_INFO_H

#include "../mm/Ref.h"

#include <boost/fusion/adapted/struct/adapt_struct.hpp>

namespace ginterpreter {

/**
 * Wrapper for thread condition ID serving as
 * an expanding area ~ an array of condition IDs.
 * It is allocated by @c MemoryState::allocExpanding()
 * and more condition IDs can be stored in
 * it with MemoryState::storeIndexed() function.
 * It is used to store IDs of threads waiting for
 * a thread condition.
 */
struct ConditionWaitingThreadsInfo
{
	int waitingThreadID;
};

/**
 * Structure specifying the layout of the values in an area
 * holding information about a mutex.
 */
struct ConditionInfo
{
	int condID;
	ConditionState condState;
	int unlockedMutexID;
	size_t waitingThreadsCount;
	mmodule::Ref<ConditionWaitingThreadsInfo>  waitingThreads;
};

/**
 * Wrapper for reference to the ConditionInfo structure
 * serving as an expanding area ~ an array of conditions.
 * It is allocated by @c MemoryState::allocExpanding()
 * and ConditionInfos for more conditions can be stored in
 * it with MemoryState::storeIndexed() function.
 */
struct ConditionsInfo
{
	mmodule::Ref<ConditionInfo> conditions;
};

}

#endif /* INT_CONDITION_INFO_H */
