// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file ConstantsManager.cpp
 * Definitions for class that manages constants in the interpreted program.
 */

#include "ConstantsManager.h"

#include "CstToValueConverter.h"

#include "../mm/MemoryState.h"
#include "../mm/Content.h"

#include <limits>

using namespace mmodule;
using namespace giterator;


namespace ginterpreter {

ConstantsManager::ConstantsManager (MemoryState* memoryState)
: m_memoryState (memoryState)
{
	m_roAreaPtr = m_memoryState->allocExpanding();
	m_nextFreeRoAreaPtr = m_roAreaPtr;
}

Ptr 
ConstantsManager::getCstPtr (const Constant* constant) const
{
    CstPtrs::const_iterator it = m_cstPtrs.find (constant);
    
    if (it != m_cstPtrs.end ())
        return it->second;
    else
        return Ptr ();
}
    
Ptr 
ConstantsManager::addCst (const Constant* constant)
{
    assert (m_cstPtrs.find (constant) == m_cstPtrs.end ());
    
    CstToValueConverter cstConv;
    constant->accept (cstConv);
    std::auto_ptr<Content> cstValue (cstConv.acquireValue ());
    
    Ptr cstPtr = m_nextFreeRoAreaPtr;
    m_nextFreeRoAreaPtr += cstValue->getUnitSize ();
    m_memoryState->store(cstPtr, cstValue);
    m_cstPtrs.insert (std::make_pair (constant, cstPtr));
    
    return cstPtr;
}

Ptr 
ConstantsManager::getFuncPtr (const Decl* decl) const
{
    DeclPtrs::const_iterator it = m_declPtrs.find (decl);
    
    if (it != m_declPtrs.end ())
        return it->second;
    else
        return Ptr ();
}

Ptr 
ConstantsManager::addFunc (const Decl* decl, std::auto_ptr<mmodule::Content> value)
{
    assert (m_declPtrs.find (decl) == m_declPtrs.end ());

    Ptr valPtr = m_nextFreeRoAreaPtr;
    m_nextFreeRoAreaPtr += value->getUnitSize();
    m_memoryState->store(valPtr, value);
    m_declPtrs.insert (std::make_pair (decl, valPtr));
    
    return valPtr;
}

Ptr
ConstantsManager::getRoAreaPtr () const
{
	return m_roAreaPtr;
}

std::pair<ConstantsManager::DeclPtrsIterator, ConstantsManager::DeclPtrsIterator>
ConstantsManager::funcs() const
{
	return std::make_pair(m_declPtrs.begin(), m_declPtrs.end());
}

}
