// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file CstToValueConverter.h
 * Class converting constants into values.
 */
#ifndef INT_CST_TO_VALUE_CONVERTER_H
#define INT_CST_TO_VALUE_CONVERTER_H

#include "../git/Constants.h"

#include "IntegerValue.h"
#include "EnumeralValue.h"
#include "RealValue.h"
#include "StringValue.h"

#include <memory>


namespace ginterpreter {
    
class Value;

/**
 * Class converting constants into values.
 * If a constant @c c needs to be converted into value, the method @c Constant::accept() 
 * is called with an object of this class as a parameter.
 * The value created from the constant @c c can be acquired by call to @c acquireValue().
 */
class CstToValueConverter : public giterator::Constant::Visitor {
  private:
    /*! converted value */
    std::auto_ptr<Value> m_value;
    
  public:
    /**
     * Acquires the converted value.
     */
    std::auto_ptr<Value> acquireValue () 
    { return m_value; }
      
    void visit (const giterator::IntegerConstant& cst)
    {
        m_value.reset (new IntegerValue (cst.getType(), cst.getValue()));
    }
    
    void visit (const giterator::EnumeralConstant& cst)
	{
		m_value.reset (new EnumeralValue (cst.getType(), cst.getValue()));
	}

    void visit (const giterator::RealConstant& cst)
	{
		m_value.reset (new RealValue (cst.getType(), cst.getValue()));
	}

    void visit (const giterator::StringConstant& cst)
    {
        m_value.reset (new StringValue (cst.getType(), cst.getValue()));
    }
};

}

#endif /* INT_CST_TO_VALUE_CONVERTER_H */
