// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file EnumeralValue.h
 * Declaration of class representing simulated enumeral value.
 */
#ifndef INT_ENUMERAL_VALUE_H
#define INT_ENUMERAL_VALUE_H

#include "IntegerValue.h"

#include "../git/EnumType.h"

namespace ginterpreter {

/**
 * Class representing simulated enumeral value.
 * It is similar to integer value, except it
 * has @c EnumType.
 */
class EnumeralValue : public IntegerValue {
  public:
    /**
     * Creates new simulated value.
     *
     * @param type type of the simulated value
     * @param value value of the simulated value
     */
    EnumeralValue (const giterator::Type* type, uint64_t value);

    /**
	 * Clone this value.
	 *
	 * @returns an exact copy of this value.
	 */
    Value* clone () const;

    /**
     * @name Arithmetic operators
     * The arithmetic operators on the value.
     */
	//@{
    std::auto_ptr<Value> addOp (const Value& v) const;
    std::auto_ptr<Value> subOp (const Value& v) const;
    std::auto_ptr<Value> multOp (const Value& v) const;

    std::auto_ptr<Value> truncDivOp (const Value& v) const;
    std::auto_ptr<Value> truncModOp (const Value& v) const;
    std::auto_ptr<Value> absOp () const;
    //@}

    /** @name Relation operators
	 * The relation operators on the value.
	 */
    //@{
    bool operator== (const Value& v) const;
    bool operator< (const Value& v) const;
    //@}

    /**
     * Print the value to a stream.
     *
     * @param os stream to print the value to
     */
    void print (std::ostream& os) const;

    /**
     * Returns the enumeral type of this simulated value.
     */
    const giterator::EnumType* getEnumeralType () const;
  protected:
	/**
	* Checks if the type of value @c i and the type of this value correspond.
	*
	* @param e the value to check if the type is the same
	*
	* @see giterator::EnumType::isSame()
	*/
	void checkType (const Value& e) const;
	/**
	* Casts the reference to the value @c v to the reference to the enumeral value
	* and checks, if the value @c v is of the same type as this value.
	*
	* @param v the value to cast
	* @return the casted reference
	*/
	const EnumeralValue& checkedCast (const Value& v) const;

};

}

#endif /* INT_ENUMERAL_VALUE_H */
