// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file FrameInfo.h
 * Structure specifying the layout of the values in an area
 * holding information about stack frame.
 */
#ifndef INT_FRAME_INFO_H
#define INT_FRAME_INFO_H

#include "../mm/Ref.h"
#include "../mm/Ptr.h"
#include "InstructionPtr.h"

#include <boost/fusion/adapted/struct/adapt_struct.hpp>

namespace ginterpreter {

/**
 * Structure specifying the layout of the values in an area 
 * holding information about stack frame.
 */
struct FrameInfo {
    /*! reference to the previous frame */
    mmodule::Ref<FrameInfo> prevFrame;
    /*! reference to the next frame */
    mmodule::Ref<FrameInfo> nextFrame;
    /*! return address */
    InstructionPtr returnAddress;
    /*! pointer to the beginning of the area holding function local variables and parameters */
    mmodule::Ptr functionLocalData;
};

}

#endif /* INT_FRAME_INFO_H */
