// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file FunctionValue.cpp
 * Definitions for class representing simulated function.
 */
#include "FunctionValue.h"

#include "../git/Function.h"
#include "../git/FunctionType.h"

#include <memory>
#include <boost/functional/hash.hpp>

namespace ginterpreter
{

FunctionValue::FunctionValue (const giterator::Function* function)
	: AbstractNonPtrValue (function->getFunctionType ()), m_function (function),
	  m_varsMapper (m_function)
{}

Value* FunctionValue::clone () const
{
	abort ();
}

mmodule::Hash FunctionValue::getHash () const
{
	return boost::hash<const giterator::Function*> () (m_function);
}

const giterator::Function* FunctionValue::getFunction () const
{
	return m_function;
}

const FunctionVarsMapper* FunctionValue::getVarsMapper () const
{
	return &m_varsMapper;
}

}
