// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file FunctionValue.h
 * Declaration of class representing simulated function.
 */
#ifndef INT_FUNCTION_VALUE_H
#define INT_FUNCTION_VALUE_H

#include "AbstractValue.h"
#include "FunctionVarsMapper.h"

namespace giterator
{
class Function;
class FunctionType;
}

namespace ginterpreter {
    
/**
 * Class representing simulated user-defined function.
 * This class is a regular simulated value, so it can be stored in the simulated memory.
 * It stores all the information needed to interpret the function.
 */
class FunctionValue : public AbstractNonPtrValue {
  private: 
    /*! GIMPLE++ representation of the body of this function */
    const giterator::Function* m_function;
    /*! mapper that maps local variables to the offsets in stack frame */
    FunctionVarsMapper m_varsMapper;
  public:
    /**
     * Constructor.
     * 
     * @param function GIMPLE++ representation of the function
     */
    FunctionValue (const giterator::Function* function);
    
    /**
	 * Clone this value.
	 * Aborts, function values should not be copied.
	 */
    Value* clone () const;
    
    /**
	 * Get a hash of this value.
	 * For memory state equivalence check purposes.
	 * Hashes the code of this function.
	 *
	 * @return hash of this value.
	 */
    mmodule::Hash getHash () const;
    
    /**
     * Returns the GIMPLE++ representation of the body of this function.
     *
     * @return the GIMPLE++ representation of the body of this function
     */
    const giterator::Function* getFunction () const;
    /**
     * Returns the mapper that maps local variables to the offsets in stack frame.
     *
     * @return the mapper that maps local variables to the offsets in stack frame
     */
    const FunctionVarsMapper* getVarsMapper () const;
};

}

#endif /* INT_FUNCTION_VALUE_H */
