// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file FunctionVarsMapper.cpp
 * Definitions for class mapping function variables to offsets.
 */
#include "FunctionVarsMapper.h"

#include "../git/Function.h"
#include "../git/FunctionType.h"

#include <boost/lambda/lambda.hpp>
#include <boost/lambda/bind.hpp>
#include <boost/lambda/casts.hpp>
#include <boost/lambda/if.hpp>
#include <boost/ref.hpp>

#include <typeinfo>

using namespace boost::lambda;
using namespace boost;
using namespace giterator;

namespace ginterpreter {

/**
 * Functor that checks whether the pointer to @c Base points to the object of @c Derived type.
 */
template <class Base, class Derived>
struct IsDynType {
    typedef bool result_type;
    
    bool operator() (const Base* b) const
    {
        return typeid (*b) == typeid (Derived);
    }
};


/**
 * Calls functor @c f for every parameter of function @c function and then for every
 * local variable declaration in function @c function.
 */
template <typename Func>
static void
forEachParamAndVar (FunctionVarsMapper* fvp, const Function* function, Func& f)
{
    const Function::Declarations& paramDecls = function->getParamDeclarations ();
    const Function::Declarations& localDecls = function->getLocalDeclarations ();
      
    f = std::for_each (paramDecls.begin (), paramDecls.end (), f);
    std::for_each (localDecls.begin (), localDecls.end (), f);

    /*
    std::for_each (localDecls.begin (), localDecls.end (),
				   if_then (bind (IsDynType<Decl, VarDecl> (), _1), f));
	*/
}


/**
 * Creates a new mapper for parameters, local variables and return value of function @c func.
 * 
 */
FunctionVarsMapper::FunctionVarsMapper (const Function* func) 
{
    m_retValSize = func->getFunctionType ()->getReturnType ()->getUnitSize ();
    forEachParamAndVar (this, func, bind (&FunctionVarsMapper::addVariable, this, _1));
}

size_t FunctionVarsMapper::getVarOffset (unsigned int uid) const
{
	return AbstractVarsMapper::getVarOffset (uid) + m_retValSize;
}

/**
 * Returns the offset where is stored the function return value.
 */
size_t FunctionVarsMapper::getRetValOffset () const
{
	return 0;
}

/**
 * Returns the size of this mapping.
 */
size_t FunctionVarsMapper::getSize () const
{
	return AbstractVarsMapper::getSize () + m_retValSize;
}

}
