// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file FunctionVarsMapper.h
 * Declaration of class mapping function variables to offsets.
 */
#ifndef INT_FUNCTION_VARS_MAPPER_H
#define INT_FUNCTION_VARS_MAPPER_H

#include "AbstractVarsMapper.h"

namespace giterator
{
class Function;
}

namespace ginterpreter {

/**
 * Class representing mapping of function local variables to the offsets within the stack frame.
 */
class FunctionVarsMapper : public AbstractVarsMapper {
  private:
    /*! size of the return value of the function this mapping is for */
    size_t m_retValSize;

  public:
    FunctionVarsMapper (const giterator::Function* func);
    
    /**
     * Returns the offset of parameter or local variable.
     * 
     * @param uid unique ID of the declaration of the parameter or local variable
     * @return offset within the stack frame
     */
    size_t getVarOffset (unsigned int uid) const;

    /**
     * Returns the offset where is stored the function return value.
     */
    size_t getRetValOffset () const;
    
    /**
     * Returns the size of this mapping.
     */
    size_t getSize () const;
    
  private:
    FunctionVarsMapper ();
};

}

#endif /* INT_FUNCTION_VARS_MAPPER_H */
