// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file GimpleDumper.h
 * Declaration of a class for dumping GIMPLE++ representation of a program into a file or stream.
 */

#ifndef INT_GIMPLE_DUMPER_H
#define INT_GIMPLE_DUMPER_H

#include <iostream>

namespace mmodule
{
class MemoryState;
}

namespace ginterpreter
{

class ConstantsManager;

/**
 * Class for dumping GIMPLE++ representation of a program into a file or stream.
 * Is serves as an alternative for the use of --fdump-tree-gimple-raw
 * option in GCC, which dumps GIMPLE representation.
 * It is an ugly dump, is should be updated.
 */
class GimpleDumper
{
private:
	/*! Constants of the simulated program. */
	ConstantsManager* m_cstManager;
	/*! Interface to the simulated memory */
	mmodule::MemoryState* m_memoryState;
public:
	/** Constructor, creating new dumper for GIMPLE++ representation
	 * of a program.
	 *
	 * @param cstManager constants of the simulated program
	 * @param memoryState interface to the simulated memory
	 */
	GimpleDumper(ConstantsManager* cstManager, mmodule::MemoryState* memoryState);

	/**
	 * Dump GIMPLE++ representation of a program to a specified file.
	 *
	 * @param filename name of a file to dump the representation to
	 */
	void dump(const char* filename);
	/**
	 * Dump GIMPLE++ representation of a program to a specified output stream.
	 *
	 * @param ostream reference to a stream to dump the representation to
	 */
	void dump(std::ostream& ostream);
};

}


#endif /* INT_GIMPLE_DUMPER_H */
