// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file GlobalsManager.cpp
 * Definitions for class that manages globals in the interpreted program.
 */
#include "GlobalsManager.h"

#include "../mm/MemoryState.h"
#include "../mm/Content.h"
#include "../Error.h"
#include "../git/Decl.h"
#include "../git/Type.h"

#include <limits>
#include <cassert>

using namespace mmodule;
using namespace giterator;


namespace ginterpreter {

/**
 * Constructor.
 */
GlobalsManager::GlobalsManager (MemoryState* memoryState)
: m_memoryState (memoryState), m_globAreaPtr(), m_nextFreeOffset(0),
  m_areaCreated(false)
{
}

Ptr GlobalsManager::getGlobalDeclPtr (const Decl* decl) const
{
	DeclPtrs::const_iterator it = m_globalDeclPtrs.find (decl);

	if (m_areaCreated && it != m_globalDeclPtrs.end ())
		return m_globAreaPtr + it->second;
	else
		return Ptr ();
}

size_t GlobalsManager::getGlobalDeclOffset (const Decl* decl) const
{
	DeclPtrs::const_iterator it = m_globalDeclPtrs.find (decl);

	if (m_areaCreated && it != m_globalDeclPtrs.end ())
		return it->second;
	else
		return 0;
}

void GlobalsManager::addGlobalDecl (const Decl* decl)
{
	assert (m_globalDeclPtrs.find (decl) == m_globalDeclPtrs.end ());

	m_globalDeclPtrs.insert (std::make_pair (decl, m_nextFreeOffset));
	m_nextFreeOffset += decl->getType()->getUnitSize();
}

Ptr GlobalsManager::getGlobAreaPtr () const
{
	return m_globAreaPtr;
}

void GlobalsManager::allocArea ()
{
	m_globAreaPtr = m_memoryState->alloc(m_nextFreeOffset);
	m_areaCreated = true;
}

}
