// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file GlobalsResolver.cpp
 * Definitions for class for finding out whether operands access global data.
 */

#include "GlobalsResolver.h"

#include "../git/Operands.h"
#include "../git/Decls.h"

#include <cassert>

using namespace giterator;

namespace ginterpreter
{

GlobalsResolver::GlobalsResolver() : m_valid(false) {}

bool GlobalsResolver::isGlobal() const
{
	assert(m_valid);
	return m_isGlobal;
}

bool GlobalsResolver::isValid() const
{
	return m_valid;
}

void
GlobalsResolver::visit(const AddrOperand& op)
{
	op.getSubOperand ()->accept(*this);
}

void
GlobalsResolver::visit(const DeclOperand& op)
{
	op.getDecl()->accept(*this);
}

void
GlobalsResolver::visit(const LabelExprOperand& op)
{
	op.getLower()->accept(*this);
	if (!m_valid && op.getIsRange())
		op.getUpper()->accept(*this);
}

void GlobalsResolver::visit (const ArrayConstructorOperand& op)
{
	// don't have to do anything, this is invalid
}

void GlobalsResolver::visit (const RecordConstructorOperand& op)
{
	// don't have to do anything, this is invalid
}

void
GlobalsResolver::visit(const IndirectRefOperand& op)
{
	op.getSubOperand ()->accept (*this);
}


void
GlobalsResolver::visit(const ArrayRefOperand& op)
{
    op.getArray ()->accept (*this);
}

void
GlobalsResolver::visit(const ComponentRefOperand& op)
{
	op.getRecord ()->accept (*this);
}


void
GlobalsResolver::visit(const CstOperand& op)
{
	// don't have to do anything, this is invalid
}

void GlobalsResolver::visit (const ParamDecl& decl)
{

}

void GlobalsResolver::visit (const VarDecl& decl)
{
	m_valid = true;
	if (decl.getLocal())
		m_isGlobal = false;
	else
		m_isGlobal = true;
}

void GlobalsResolver::visit (const RecordDecl& decl)
{
	m_valid = true;
	if (decl.getLocal())
		m_isGlobal = false;
	else
		m_isGlobal = true;
}

void GlobalsResolver::visit (const FunctionDecl& decl)
{

}

void GlobalsResolver::visit (const FieldDecl& decl)
{

}

void GlobalsResolver::visit (const LabelDecl& decl)
{

}

void GlobalsResolver::visit (const ResultDecl& decl)
{

}

void GlobalsResolver::visit (const TypeDecl& decl)
{

}

void GlobalsResolver::visit (const ConstDecl& decl)
{

}


}
