// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file GlobalsResolver.h
 * Declaration of a class for finding out whether operands access global data.
 */

#ifndef INT_GLOBALS_RESOLVER_H
#define INT_GLOBALS_RESOLVER_H

#include "../git/Operand.h"
#include "../git/Decl.h"

namespace ginterpreter
{

/**
 * Class for classifying declarations to global and local
 * Globla operands are these that access the global memory.
 */
class GlobalsResolver : public giterator::Operand::Visitor, public giterator::Decl::Visitor
{
private:
	/*! Holder for the desired result - whether the asked operand is global. */
	bool m_isGlobal;
	/*! Whether it was valid to asked about globality of operand. */
	bool m_valid;
public:
	/**
	 * Constructor.
	 */
	GlobalsResolver();

	/**
	 * Find out whether operand we asked about was global or local.
	 * Always call this after asking isValid() == true.
	 *
	 * @return true if the asked operand was global, false otherwise
	 *
	 * @see isValid()
	 */
	bool isGlobal() const;
	/**
	 * Find out whether it was valid to ask about globality of operand ~
	 * whether there was a variable declaration (VarDecl).
	 *
	 * @return true if it was valid to ask about globality of operand,
	 * false otherwise
	 */
	bool isValid () const;

	void visit (const giterator::AddrOperand&);
	void visit (const giterator::DeclOperand&);
	void visit (const giterator::LabelExprOperand&);
	void visit (const giterator::IndirectRefOperand&);
	void visit (const giterator::ArrayRefOperand&);
	void visit (const giterator::ComponentRefOperand&);
	void visit (const giterator::CstOperand&);
	void visit (const giterator::ArrayConstructorOperand&);
	void visit (const giterator::RecordConstructorOperand&);

	void visit (const giterator::ParamDecl& decl);
	void visit (const giterator::VarDecl& decl);
	void visit (const giterator::RecordDecl& decl);
	void visit (const giterator::FunctionDecl& decl);
	void visit (const giterator::FieldDecl& decl);
	void visit (const giterator::LabelDecl& decl);
	void visit (const giterator::ResultDecl& decl);
	void visit (const giterator::TypeDecl& decl);
	void visit (const giterator::ConstDecl& decl);
};

}

#endif /* INT_GLOBALS_RESOLVER_H */
