// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file InstructionPtr.h
 * Declaration of class representing instruction pointer.
 */
#ifndef INT_INSTRUCTION_PTR_H
#define INT_INSTRUCTION_PTR_H

#include "../git/Block.h"
#include "../git/git_fwd.h"

#include <boost/functional/hash.hpp>


namespace ginterpreter {

class FunctionValue;


/**
 * Class representing instruction pointer (IP).
 * Instruction pointer points to a GIMPLE++ statement.
 */
class InstructionPtr {
    /*! function to which point this IP */
    const FunctionValue* m_func;
    /*! block within function @c m_func to which point this IP */
    const giterator::Block* m_block;
    /*! statement within block @c m_block to which point this IP */
    giterator::Block::StmtIterator m_stmtIter;
    
  public:
    InstructionPtr (const FunctionValue* func);
    InstructionPtr ();
    
    const FunctionValue* getFunction () const;
    const giterator::Block* getBlock () const;
    const giterator::Stmt* getStmt () const;
        
    void step ();
    void jump ();
    void takeBranch (bool cond);
    void jumpToLabel (const giterator::Operand* label);
    
    /**
     * Checks if this IP is null.
     */
    bool isNull () const;
    
};


/**
 * Returns the hash value of the instruction pointer @c ip.
 */
inline std::size_t 
hash_value (const InstructionPtr& ip)
{
    if (ip.isNull ())
        return 0;
    
    std::size_t seed = 0;
    
    boost::hash_combine (seed, ip.getFunction ());
    boost::hash_combine (seed, ip.getBlock ());
    boost::hash_combine (seed, ip.getStmt ());
    
    return seed;
}

}

#endif /* INT_INSTRUCTION_PTR_H */
