// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file Interpret.h
 * Declarations of functions for GIMPLE interpretation callable from C.
 */
#ifndef INT_INTERPRET_H
#define INT_INTERPRET_H


#include "InterpretFlags.h"

#ifdef __cplusplus
extern "C" {
namespace ginterpreter {
#endif

/**
 * Runs the interpreter.
 * This function interprets the program step by step and the thread scheduling policy is determined
 * by @c ThreadPool class. The policy is that a thread is selected to run after a previously selected
 * thread is finished or its state is not running (e.g. it is waiting for another thread to join,
 * waiting for a mutex or for a thread condition).
 *
 * During interpretation, visited states are not stored, therefore no state equivalence matching is
 * done. Following pseudocode shows how the interpreter algorithm works:
 * @code
 * program.init();
 * while(!program.isFinished())
 * {
 *  program.step();
 *  if (!program.currentThread().runnable())
 *    program.selectRunnableThread();
 * }
 * @endcode
 *
 * In the interpreter, deadlocks and data race problems do not have to be detected.
 *
 * @param flags flags for the interpreter determining its behavior (unused)
 * @param gimplexx_file name of a file where the GIMPLE++ representation of a program is
 * @return 0 if successful, -1 otherwise
 * serialized
 */
extern int doInterpret (struct interpreter_flags flags, const char* gimplexx_file);

#ifdef __cplusplus
}}
#endif
    
#endif /* INT_INTERPRET_H */

