// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file MemoryContentPrinter.cpp
 * Definitions for printer that prints the actual values
 * of declaration operands.
 */
#include "MemoryContentPrinter.h"

#include "ValueReader.h"
#include "AddrResolver.h"
#include "OstreamOperators.h"

#include "../git/DeclOperand.h"
#include "../git/ArrayType.h"
#include "../mm/MemoryExceptions.h"

#include <ostream>

using namespace giterator;


namespace ginterpreter {

MemoryContentPrinter::MemoryContentPrinter (std::ostream& os, const FunctionValue* func,
                          const mmodule::MemoryState* memoryState,
                          const ConstantsManager* cstManager,
                          const GlobalsManager* globalsManager,
                          mmodule::Ptr frame)
: PrinterBase (os), m_func (func), m_memoryState (memoryState),
  m_cstManager (cstManager), m_globalsManager(globalsManager),
  m_frame (frame)
{}

/**
 * Prints the value of the variable represented by @c op.
 */
void
MemoryContentPrinter::visit(const DeclOperand& op)
{
    const ArrayType* arrayType = dynamic_cast<const ArrayType*> (op.getType ());
    if (arrayType)
    {
        AddrResolver addrRes (m_func, m_memoryState, m_cstManager, m_globalsManager, m_frame);
        op.accept (addrRes);
        m_os << addrRes.getPtr ();
    }
    else
    {
        ValueReader reader (m_func, m_memoryState, m_cstManager, m_globalsManager, m_frame);
        
        try {
            op.accept (reader);
            m_os << *reader.getValue ();
        } catch (mmodule::UndefinedMemoryLoadException& e) {
            m_os << "N/A";
        }
    }
}

}
