// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file MemoryContentPrinter.h
 * Declaration of printer that prints the actual values
 * of declaration operands.
 */
#ifndef INT_MEMORY_CONTENT_PRINTER_H
#define INT_MEMORY_CONTENT_PRINTER_H

#include "PrinterBase.h"

#include "../mm/fwd.h"
#include "../mm/Ptr.h"


namespace ginterpreter {
    
class FunctionValue;
class ConstantsManager;
class GlobalsManager;
    
/**
 * Printer that prints the actual values of the declaration operands 
 * rather than printing them only symbolically.
 */
class MemoryContentPrinter : public PrinterBase 
{
    const FunctionValue* m_func;
    const mmodule::MemoryState* m_memoryState;
    const ConstantsManager* m_cstManager;
    const GlobalsManager* m_globalsManager;
    mmodule::Ptr m_frame;
    
  public:
    /**
     * Creates new content printer.
     * 
     * @param os output stream where to print the value
     * @param func current function
     * @param memoryState memory state
     * @param cstManager constants manager
     * @param globalsManager globals manager
     * @param frame pointer to the current stack frame
     */
    MemoryContentPrinter (std::ostream& os, const FunctionValue* func, 
                          const mmodule::MemoryState* memoryState, 
                          const ConstantsManager* cstManager,
                          const GlobalsManager* globalsManager,
                          mmodule::Ptr frame);
    
    void visit(const giterator::DeclOperand& op);
};

}

#endif /* INT_MEMORY_CONTENT_PRINTER_H */
