// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file ModelCheckerExceptions.h
 * Exceptions that can be thrown by model chekcer.
 */
#ifndef INT_MODEL_CHECKER_EXCEPTIONS_H
#define INT_MODEL_CHECKER_EXCEPTIONS_H

#include <exception>

namespace ginterpreter {

/**
 * Common base class of all exceptions that can be thrown when
 * working with the model checker.
 */
struct ModelCheckerException : public std::exception {
};

/**
 * Exception that is thrown when deadlock occured.
 * This is the case when model checker detected a deadlock.
 * Interpreter can detect som deadlocks as well.
 * @see IDeadlockException.
 */
struct MCDeadlockException : public ModelCheckerException {
    /**
     * Constructor.
     */
	MCDeadlockException() {}

    const char* what () const throw ()
    {
        return "Deadlock!";
    }
};

/**
 * Exception that is thrown when exit was called on last (or only) thread of
 * program.
 */
struct ThreadExitException : public ModelCheckerException {
	ThreadExitException(long int ID) : m_ID(ID) {}

	const char* what () const throw()
	{
		return "Exit called on thread.";
	}

	long int getId() const
	{ return m_ID; }
private:
	long int m_ID;
};

/**
 * Exception that is thrown when exit was called on last (= only) thread of
 * program.
 */
struct LastThreadExitException : public ModelCheckerException {
	LastThreadExitException(long int ID) : m_ID(ID) {}

	const char* what () const throw()
	{
		return "Exit called on last thread of program.";
	}
	long int getId() const
	{ return m_ID; }
private:
	long int m_ID;
};

}

#endif /* INT_INTERPRET_EXCEPTIONS_H */
