// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file MutexInfo.h
 * Structure specifying the layout of the values in an area
 * holding information about a mutex.
 */
#ifndef INT_MUTEX_INFO_H
#define INT_MUTEX_INFO_H

#include "../mm/Ref.h"

#include <boost/fusion/adapted/struct/adapt_struct.hpp>

namespace ginterpreter {

/**
 * Wrapper for mutex ID serving as an expanding area
 * ~ an array of mutex IDs.
 * It is allocated by @c MemoryState::allocExpanding()
 * and more mutex IDs can be stored in
 * it with MemoryState::storeIndexed() function.
 * It is used to store IDs of threads waiting to
 * acquire a lock on a mutex.
 */
struct MutexWaitingThreadsInfo
{
	int waitingThreadID;
};

/**
 * Structure specifying the layout of the values in an area
 * holding information about a mutex.
 */
struct MutexInfo
{
	int mutexID;
	int ownerID;
	MutexState mutexState;
	bool isReferenced;
	size_t waitingThreadsCount;
	mmodule::Ref<MutexWaitingThreadsInfo>  waitingThreads;
};

/**
 * Wrapper for reference to the MutexInfo structure
 * serving as an expanding area ~ an array of mutexes.
 * It is allocated by @c MemoryState::allocExpanding()
 * and MutexInfos for more mutexes can be stored in
 * it with MemoryState::storeIndexed() function.
 */
struct MutexesInfo
{
	mmodule::Ref<MutexInfo> mutexes;
};

}

#endif /* INT_MUTEX_INFO_H */
