// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file NameResolver.cpp
 * Definitions for class for resolving names of operands.
 */

#include "NameResolver.h"

#include "../git/Operands.h"

#include <cassert>

using namespace giterator;

namespace ginterpreter
{

NameResolver::NameResolver() {}

std::string NameResolver::getName() const
{
	assert(m_name.compare("") != 0);
	return m_name;
}

void
NameResolver::visit(const AddrOperand& op)
{
	op.getSubOperand ()->accept(*this);
}

void
NameResolver::visit(const DeclOperand& op)
{
    m_name = op.getDecl()->getName();
}

void
NameResolver::visit(const LabelExprOperand& op)
{
	assert (false);
}

void NameResolver::visit (const ArrayConstructorOperand& op)
{
	assert (false);
}

void NameResolver::visit (const RecordConstructorOperand& op)
{
	assert (false);
}

void
NameResolver::visit(const IndirectRefOperand& op)
{
	op.getSubOperand ()->accept (*this);
}


void
NameResolver::visit(const ArrayRefOperand& op)
{
    op.getArray ()->accept (*this);
}

void
NameResolver::visit(const ComponentRefOperand& op)
{
	op.getField ()->accept (*this);
}


void
NameResolver::visit(const CstOperand& op)
{
	assert(false);
}

}
