// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file NameResolver.h
 * Declaration of a class for resolving names of operands.
 */

#ifndef INT_NAME_RESOLVER_H
#define INT_NAME_RESOLVER_H

#include "../git/Operand.h"

#include <string>

namespace ginterpreter
{

/**
 * Class for resolving names of declaration operands.
 *
 * Used to obtain names of thread condition variables
 * that are stored inside @c ThreadNameHolder class inside
 * @c ThreadPool class.
 * These names are used only for output purposes.
 *
 * @see ThreadNameHolder
 */
class NameResolver : public giterator::Operand::Visitor
{
private:
	/*! The found name of an operand */
	std::string m_name;
public:
	/**
	 * Constructor, creates a new name resolver.
	 */
	NameResolver();

	/**
	 * Obtain the found name.
	 *
	 * @return name of an operand this class visited
	 *
	 */
	std::string getName() const;

	void visit (const giterator::AddrOperand&);
	void visit (const giterator::DeclOperand&);
	void visit (const giterator::LabelExprOperand&);
	void visit (const giterator::IndirectRefOperand&);
	void visit (const giterator::ArrayRefOperand&);
	void visit (const giterator::ComponentRefOperand&);
	void visit (const giterator::CstOperand&);
	void visit (const giterator::ArrayConstructorOperand&);
	void visit (const giterator::RecordConstructorOperand&);
};

}

#endif /* INT_NAME_RESOLVER_H */
