// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file OperationInterpreter.h
 * Declaration of class that interprets GIMPLE++ operations.
 */
#ifndef INT_OPERATION_INTERPRETER_H
#define INT_OPERATION_INTERPRETER_H

#include "../git/git_fwd.h"
#include "../git/Operation.h"
#include "../git/Operations_fwd.h"
#include "../mm/fwd.h"
#include "../mm/Ptr.h"

#include "Value.h"

#include <memory>
#include <boost/noncopyable.hpp>

namespace ginterpreter {

class FunctionValue;
class ConstantsManager;
class GlobalsManager;


/**
 * Class that allows to interpret GIMPLE++ operations.
 */
class OperationInterpreter 
    : public giterator::Operation::Visitor, private boost::noncopyable 
{
  private:
    const FunctionValue* m_func;
    mmodule::MemoryState* m_memoryState;
    ConstantsManager* m_cstManager;
    GlobalsManager* m_globalsManager;
    
    giterator::Gimple* m_gimple;
    mmodule::Ptr m_frame;
    
    /*! holds the result of the operation */
    std::auto_ptr<Value> m_result;
        
  public:
    /**
     * Constructor.
     */
    OperationInterpreter (const FunctionValue* func, mmodule::MemoryState* memoryState, 
                          ConstantsManager* cstManager,
                          GlobalsManager* globalsManager,
                          giterator::Gimple* gimple,
                          mmodule::Ptr frame);
    /**
     * Acquires the result of the operation.
     */
    std::auto_ptr<Value> acquireResult ();
    
    void visit (const giterator::NegateOperation&);
    void visit (const giterator::AbsOperation&);
    void visit (const giterator::FixTruncOperation&);
    void visit (const giterator::FloatOperation&);
    void visit (const giterator::BitNotOperation&);
    void visit (const giterator::TruthNotOperation&);
    void visit (const giterator::ParenOperation&);
    void visit (const giterator::NonLvalueOperation&);
    void visit (const giterator::NoOperation&);
    void visit (const giterator::ConvertOperation&);

    void visit (const giterator::PlusOperation&);
    void visit (const giterator::MinusOperation&);
    void visit (const giterator::MultOperation&);
    void visit (const giterator::RdivOperation&);
    void visit (const giterator::TruncDivOperation&);
    void visit (const giterator::FloorDivOperation&);
    void visit (const giterator::CeilDivOperation&);
    void visit (const giterator::RoundDivOperation&);
    void visit (const giterator::TruncModOperation&);
    void visit (const giterator::FloorModOperation&);
    void visit (const giterator::CeilModOperation&);
    void visit (const giterator::RoundModOperation&);
    void visit (const giterator::ExactDivOperation&);
    void visit (const giterator::MinOperation&);
    void visit (const giterator::MaxOperation&);
    void visit (const giterator::LshiftOperation&);
    void visit (const giterator::RshiftOperation&);
    void visit (const giterator::LrotateOperation&);
    void visit (const giterator::RrotateOperation&);
    void visit (const giterator::BitIorOperation&);
    void visit (const giterator::BitXorOperation&);
    void visit (const giterator::BitAndOperation&);
    void visit (const giterator::TruthAndOperation&);
    void visit (const giterator::TruthOrOperation&);
    void visit (const giterator::TruthXorOperation&);
    void visit (const giterator::LtOperation&);
    void visit (const giterator::LeOperation&);
    void visit (const giterator::GtOperation&);
    void visit (const giterator::GeOperation&);
    void visit (const giterator::EqOperation&);
    void visit (const giterator::NeOperation&);
    void visit (const giterator::OrderedOperation&);
    void visit (const giterator::UnorderedOperation&);
    void visit (const giterator::UnLtOperation&);
    void visit (const giterator::UnLeOperation&);
    void visit (const giterator::UnGtOperation&);
    void visit (const giterator::UnGeOperation&);
    void visit (const giterator::UnEqOperation&);
    void visit (const giterator::LtGtOperation&);
    void visit (const giterator::WidenSumOperation&);
    void visit (const giterator::WidenMultOperation&);
};

}

#endif  /* INT_OPERATION_INTERPRETER_H */
