// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file PtrValue.cpp
 * Definitions for class representing simulated pointer value.
 */
#include "PtrValue.h"

#include "OstreamOperators.h"
#include "IntegerValue.h"
#include "../git/PtrType.h"

#include <typeinfo>
#include <memory>
#include <boost/functional/hash.hpp>


namespace ginterpreter
{

PtrValue::PtrValue(const giterator::Type* type, mmodule::Ptr ptr)
	: AbstractPtrValue (type), m_ptr(ptr) {}

Value* PtrValue::clone() const
{
  return new PtrValue (getType(), m_ptr);
}

mmodule::Ptr PtrValue::getPtr() const
{
  return m_ptr;
}

const giterator::PtrType* PtrValue::getPtrType() const
{
  return static_cast<const giterator::PtrType*>(getType());
}


std::auto_ptr<Value> PtrValue::addOp (const Value& v) const
{
  const IntegerValue& iv = checkedIntegerCast (v);
  return std::auto_ptr<Value> (new PtrValue (getType(), m_ptr + iv.getValue()));
}

bool PtrValue::operator< (const Value& v) const
{
  const PtrValue& pv = checkedPtrCast (v);
  return m_ptr < pv.m_ptr;
}

bool PtrValue::operator== (const Value& v) const
{
  // TODO:
  if (typeid(v) == typeid(const IntegerValue))
  {
	const IntegerValue& iv = checkedIntegerCast (v);
	if (iv.isZero () && m_ptr.isNull ())
		return true;
	else
		return false;
  }

  const PtrValue& pv = checkedPtrCast (v);
  return m_ptr == pv.m_ptr;
}

bool PtrValue::isPtr () const
{ return true; }

void PtrValue::print (std::ostream& os) const
{
	os << m_ptr;
}

const IntegerValue& PtrValue::checkedIntegerCast (const Value& v) const
{
	const IntegerValue& iv = dynamic_cast<const IntegerValue&> (v);
	assert (&iv);
	return iv;
}

const PtrValue& PtrValue::checkedPtrCast (const Value& v) const
{
	const PtrValue& pv = dynamic_cast<const PtrValue&> (v);
	assert (&pv);
	return pv;
}

}
