// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file RecordConstructorValue.cpp
 * Definitions for class representing simulated record constructor.
 */
#include "RecordConstructorValue.h"

#include <boost/functional/hash.hpp>

namespace ginterpreter
{

RecordConstructorValue::RecordConstructorValue (const giterator::Type* type)
	: AbstractNonPtrValue (type)
{}

Value* RecordConstructorValue::clone () const
{
	RecordConstructorValue* v = new RecordConstructorValue (getType ());
	for (Values::const_iterator i1 = m_values.begin();
			i1 != m_values.end();
			++i1)
	{
		//v->m_values[i1->first] = i1->second;
		v->m_values.insert(i1->first, std::auto_ptr<Value>(i1->second->clone()));
	}

	return v;
}

mmodule::Hash RecordConstructorValue::getHash () const
{
	std::size_t seed = 0;

	for(ValuesIterator i1 = m_values.begin(); i1 != m_values.end(); ++i1)
	{
		boost::hash_combine(seed, i1->second);
	}

	return seed;
}

void RecordConstructorValue::addValue(const giterator::FieldDecl* field, std::auto_ptr<Value> constructor)
{
	//m_values[field] = constructor;
	m_values.insert(field, constructor);
}

std::pair<RecordConstructorValue::ValuesIterator, RecordConstructorValue::ValuesIterator>
RecordConstructorValue::values() const
{
	return std::make_pair<ValuesIterator, ValuesIterator> (m_values.begin(), m_values.end());
}

}
