// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file StringValue.h
 * Declaration of class representing a simulated string value.
 */
#ifndef INT_STRING_VALUE_H
#define INT_STRING_VALUE_H

#include "AbstractValue.h"

namespace ginterpreter {
    
/**
 * Class representing a simulated string value.
 */
class StringValue : public AbstractNonPtrValue {
  private:
    /*! value of this simulated value */
    std::string m_value;

  public:
    /**
     * Creates new simulated string value.
     * 
     * @param type type of the simulated value
     * @param value value of the simulated value
     */
    StringValue (const giterator::Type* type, const std::string& value);
    
    /**
     * Returns the value of this simulated value.
     */
    const std::string& getValue () const;
    
    Value* clone () const;
    
    mmodule::Hash getHash () const;
    
    void print (std::ostream& os) const;
};

}

#endif /* INT_STRING_VALUE_H */
