// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file ThreadInfo.h
 * Structure specifying the layout of the values in an area
 * holding information about the thread.
 */
#ifndef INT_THREAD_INFO_H
#define INT_THREAD_INFO_H

#include "InstructionPtr.h"

#include "../mm/Ref.h"

#include <boost/fusion/adapted/struct/adapt_struct.hpp>


namespace ginterpreter {
    
class FrameInfo;

/**
 * Structure specifying the layout of the values in an area 
 * holding information about the thread.
 */
struct ThreadInfo {
    /*! current instruction pointer */
    InstructionPtr currentIp;
    /*! ID of the thread */
    int threadID;
    /*! State of thread */
    ThreadState threadState;
    /*! ID of thread the thread is waiting for */
    int waitingFor;
    /*! Whether the thread is being joined by another thread */
    bool isBeingJoined;
    /*! pointer to the bottom stack frame */
    mmodule::Ref<FrameInfo> bottomFrame;
};

}

#endif /* INT_THREAD_INFO_H */
