// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file ThreadNameHolder.h
 * Class holding map thread ID -> thread name (name of variable holding the thread, or main).
 */

#include "Thread.h"

#include <string>
#include <map>

namespace ginterpreter {

/**
 * Class for mapping IDs of threads to the names of the
 * thread variables representing them (or 'main' form main thread).
 */
class ThreadNameHolder
{
private:
	/*! type of the container for mapping IDs of threads to their names */
	typedef std::map<Thread::ID, std::string> NameMap;
public:
	/*! container mapping IDs of threads to their names */
	NameMap m_nameMap;

	/**
	 * Add a name of a thread with a specified ID to the
	 * mapping of this class.
	 *
	 * @param ID ID of the added thread
	 * @param name of the thread with ID @c ID
	 */
	void insert(Thread::ID ID, std::string name)
	{
		m_nameMap[ID] = name;
	}

	/**
	 * Get a name of a thread with specified ID.
	 */
	const std::string& getName(Thread::ID id) const
	{
		NameMap::const_iterator i1 = m_nameMap.find(id);
		assert( i1 != m_nameMap.end());
		return i1->second;
	}
};
}
