// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file ThreadPoolInfo.h
 * Structure specifying the layout of the values in an area
 * holding information about all threads.
 */
#ifndef INT_THREAD_POOL_INFO_H
#define INT_THREAD_POOL_INFO_H

#include "ThreadInfo.h"
#include "MutexInfo.h"
#include "ConditionInfo.h"
#include "../mm/Ref.h"

#include <boost/fusion/adapted/struct/adapt_struct.hpp>
#include <vector>

namespace ginterpreter {

/**
 * Wrapper for reference to the ThreadInfo structure
 * serving as an expanding area ~ an array of threads.
 * It is allocated by @c MemoryState::allocExpanding()
 * and ThreadInfos for more threads can be stored in
 * it with MemoryState::storeIndexed() function.
 */
struct ThreadsInfo
{
	mmodule::Ref<ThreadInfo> threads;
};

/**
 * Structure specifying the layout of the values in an area
 * in simulated memory  holding information about the thread
 * pool.
 *
 * @see ThreadPool
 */
struct ThreadPoolInfo {
    /*! ID of main thread of program */
    Thread::ID mainThreadID;
    /*! ID of current thread of program */
    Thread::ID currentThreadID;
    /*! current number of threads of program */
    size_t threadCount;

    /*! Variable holding IDs of all threads of the program */
    unsigned int threadIds;
    /*! Variable holding IDs of all mutexes of the program */
    unsigned int mutexIds;
    /*! Variable holding IDs of all thread conditions of the program */
    unsigned int conditionIds;
    /*! info structures of the threads of the program */
    mmodule::Ref<ThreadsInfo> threads;
    /*! info structures of the mutexes of the program */
	mmodule::Ref<MutexesInfo> mutexes;
	/*! info structures of the conditions of the program */
	mmodule::Ref<ConditionsInfo> conditions;
};

}

#endif /* INT_THREAD_POOL_INFO_H */
