// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file ThreadsManager.h
 * Declaration of class that manages threads in the interpreted program.
 */
#ifndef INT_THREADS_MANAGER_H
#define INT_THREADS_MANAGER_H

//TODO: most likely delete this file

#include "Thread.h"

#include "../mm/fwd.h"
#include "../mm/Ptr.h"

#include <map>
#include <boost/noncopyable.hpp>

namespace ginterpreter {

class ThreadInfo;

/**
 * Class that manages threads in the interpreted program.
 */
class ThreadsManager : private boost::noncopyable {
  private:
    typedef std::map<Thread::ID, mmodule::Ref<ThreadInfo> > ThreadPtrs;

    mmodule::MemoryState* m_memoryState;

    /*! map containing pointers for threads. */
    ThreadPtrs m_threadPtrs;

    /*! pointer to the beginning of the area for threads. */
    mmodule::Ptr m_threadPtr;
  public:
    ThreadsManager (mmodule::MemoryState* memoryState);

    mmodule::Ptr getThreadPtr (Thread::ID ID) const;
    mmodule::Ptr addThreadInfo (Thread::ID ID, mmodule::Ref<ThreadInfo> threadInfo );

    /**
     * Returns pointer to the beginning of the area containing all
     * threads.
     */
    mmodule::Ptr getThreadPtr () const;
};

}

#endif /* INT_GLOBALS_MANAGER_H */
