// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file Timer.cpp
 * Definitions for class counting specified periods amount of time.
 */

#include "Timer.h"

#include <cassert>

namespace ginterpreter
{

Timer::Timer() : m_started(false)
{
}

void
Timer::setInterval(unsigned int interval)
{
	assert (m_started == false);

	m_interval = interval;
}

void
Timer::start()
{
	assert (m_started == false);

	m_starttime = time (NULL);
	m_started = true;
}

bool
Timer::isStarted() const
{
	return m_started;
}

bool
Timer::elapsed() const
{
	if (!isStarted())
		return true;

	time_t currenttime = time(NULL);
	if ((unsigned int) difftime(currenttime, m_starttime) >= m_interval)
		return true;

	return false;
}

unsigned int
Timer::getRemainingTime() const
{
	if (!isStarted())
		return 0;

	time_t currenttime = time(NULL);
	double diff = difftime(m_starttime + m_interval, currenttime);

	if (diff <= 0)
		return 0;
	else
		return (unsigned int) diff;
}

void
Timer::clear()
{
	assert(elapsed());
	m_started = false;
}

}
