// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/* @file UidResolver.cpp
 * Definitions for class for resolving ID's of operands.
 */

#include "UidResolver.h"

#include "../git/Operands.h"
#include "../git/IntegerConstant.h"

#include <cassert>
#include <limits.h>

using namespace giterator;

namespace ginterpreter
{

UidResolver::UidResolver() : m_uid(Thread::NO_ID) {}

UidResolver::ID UidResolver::getUid() const
{
	assert(m_uid != Thread::NO_ID);
	return m_uid;
}

void
UidResolver::visit(const AddrOperand& op)
{
	op.getSubOperand ()->accept(*this);
}

void
UidResolver::visit(const DeclOperand& op)
{
    m_uid = op.getDecl()->getUid();
}

void
UidResolver::visit(const LabelExprOperand& op)
{
	assert (false);
}

void UidResolver::visit (const ArrayConstructorOperand& op)
{
	assert (false);
}

void UidResolver::visit (const RecordConstructorOperand& op)
{
	assert (false);
}

void
UidResolver::visit(const IndirectRefOperand& op)
{
	op.getSubOperand ()->accept (*this);
}


void
UidResolver::visit(const ArrayRefOperand& op)
{
    op.getArray ()->accept (*this);
    unsigned int uidArr = m_uid;

    op.getIndex ()->accept(*this);
    unsigned int uidInd = m_uid;

    //TODO:
    m_uid = uidArr * uidInd;
}

void
UidResolver::visit(const ComponentRefOperand& op)
{
	op.getField ()->accept (*this);
}


void
UidResolver::visit(const CstOperand& op)
{
	// this should be visited only for index of array ref operands,
	// therefore it is integer
    const IntegerConstant* ic = dynamic_cast<const IntegerConstant*> (op.getConstant());
    assert(ic);

    m_uid = ic->getValue() % UINT_MAX;
}

}
