// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

/** @file PtrValue.h
 * Declaration of class representing simulated void pointer value.
 */
#ifndef INT_VOID_PTR_VALUE_H
#define INT_VOID_PTR_VALUE_H

#include "PtrValue.h"

namespace ginterpreter {

/**
 * Class representing a simulated void pointer value.
 * This class is a regular simulated value, so it can be stored in the simulated memory.
 * In GMC however, VoidPtrValue is not stored to the simulated memory.
 * It is only a helper class that is returned from a convert operation
 * in @c OperationInterpreter when a conversion to void pointer occurs.
 */
class VoidPtrValue : public PtrValue {
   public:
    /**
     * Constructor.
     *
     * @param type type of the value
     * @param ptr pointer that the value represents
     */
    VoidPtrValue(const giterator::Type* type, mmodule::Ptr ptr);

    /**
	 * Clone this value.
	 *
	 * @returns an exact copy of this value.
	 */
    virtual Value* clone() const;
};

}

#endif /* INT_VOID_PTR_VALUE_H */
