// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

#ifndef MM_OBJECTCOLORMAP_H
#define MM_OBJECTCOLORMAP_H

#include "AreaEntry.h"

#include <boost/property_map/property_map.hpp>


namespace mmodule { namespace detail {
    
/**
 * Property map that can hold colors used in @c boost::breadth_first_search algorithm.
 * The values are stored directly in @c AreaEntry objects.
 * 
 * @see Boost Graph library documentation
 */
class AreaColorMap {
  public:
    /*! key type */
    typedef AreaEntry* key_type;
    /*! value type */
    typedef int value_type;
    /*! reference type */
    typedef value_type& reference;
    /*! category of this proprety map */
    typedef boost::read_write_property_map_tag category;
};

/**
 * Setc color of area.
 * 
 * @param ae area entry which color to set
 * @param color color
 */
inline void
put (AreaColorMap&, const AreaEntry* ae, AreaColorMap::value_type color)
{
    const_cast<AreaEntry*> (ae)->setColor (color);
}

/**
 * Gets color of area.
 * 
 * @param ae area entry which color to return
 * @return color of area @c ae
 */
inline int 
get (AreaColorMap&, const AreaEntry* ae)
{
    return ae->getColor ();
}

}}

#endif

