// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

#ifndef MM_MODIFICATIONSVISITOR_H
#define MM_MODIFICATIONSVISITOR_H


namespace mmodule { namespace detail {
    
class AreaEntry;
class ContentEntry;


/**
 * Default visitor modeling Modifications Visitor concept.
 * 
 * The requirements on the type @c V modeling Modifications Visitor are as follows:
 * 
 *<H3>Notation:</H3>
 * <TABLE>
 *    <TR>  <TD>V</TD> <TD>A type that is a model of Modifications Visitor</TD>  </TR>
 *    <TR>  <TD>vis</TD> <TD>An object of type @c V </TD>   </TR>
 *    <TR>  <TD>ae</TD> <TD>An object of type @c AreaEntry </TD>  </TR>
 *    <TR>  <TD>ce</TD> <TD>An object of type @c ContentEntry </TD>  </TR>
 * </TABLE>
 * 
 * <H3>Folowing expressions must be valid:</H3>
 * <TABLE>
 *    <TR> 
 *      <TD><b>Expression</b></TD><TD><b>Return type</b></TD>  
 *      <TD><b>Description</b></TD>
 *    </TR>
 *    <TR> 
 *      <TD>vis.beforeModifiedArea&nbsp;(ae)</TD><TD>bool</TD>  
 *      <TD>Called for every modified area before the modified values are visited.
 *          If the visitor wants to be notified about modified values in the area @c ae, 
 *          the method must return @c true. To skip the modified values return @c false.
 *      </TD>
 *    </TR>
 *    <TR> 
 *      <TD>vis.afterModifiedArea&nbsp;(ae)</TD><TD>void</TD>   
 *      <TD>Called for every modified area after the modified values were visited.
 *          This function is called for area @c ae if and only if @c beforeModifiedArea(ae) 
 *          returned @c true.
 *      </TD>
 *    </TR>
 *    <TR> 
 *      <TD>vis.modifiedContent&nbsp;(ae,&nbsp;ce)</TD><TD>void</TD> 
 *      <TD>Called for every modified value of the area.</TD>
 *    </TR>
 * </TABLE>
 * 
 * @see State::walkModifications()
 */
struct DefaultModificationsVisitor {
    /**
     * Returns @c true.
     */
    bool beforeModifiedArea (AreaEntry*)
    { return true; }
    
    /**
     * Does nothing.
     */
    void afterModifiedArea (AreaEntry*) 
    {}
    
    /**
     * Does nothing.
     */
    void modifiedContent (AreaEntry*, ContentEntry*)
    {}
};

}}

#endif
