// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

#ifndef MM_HASHHOLDER_H
#define MM_HASHHOLDER_H

#include "Types.h"

#include <cstdlib>


namespace mmodule { namespace detail {
    
class RandomValues;


/**
 * Class that holds and updates hash of the state of the memory.
 */
class HashHolder {
    /*! state hash */
    Hash m_hash;
    
    /*! 
     * functor returning random values for canonincal addresses used 
     * in area partial hash computation 
     */
    RandomValues* m_areaRand;
    
    /*! 
     * functor returning random values for canonincal addresses used 
     * in value partial hash computation 
     */
    RandomValues* m_valueRand;
    
  public:
    HashHolder (RandomValues* areaRand, RandomValues* valueRand);
    ~HashHolder (); 
       
    void addContentHash (const CanonAddr& addr, const Hash& contentHash);
    void removeContentHash (const CanonAddr& oldAddr, const Hash& oldContentHash);
    
    void addAreaHash (const CanonAddr& areaBeginAddress, size_t areaSize);
    void removeAreaHash (const CanonAddr& areaBeginAddress, size_t areaSize);
    
    /**
     * Returns state hash.
     * 
     * @return state hash
     */
    Hash getHash () const
    { return m_hash; }
    
    /**
     * Sets state hash.
     * 
     * @param hash new state hash
     */
    void setHash (Hash hash) 
    { m_hash = hash; }
    
  private:
    void addPartialHash (Hash partialHash);
    void removePartialHash (Hash partialHash);
    
    Hash computeContentPartialHash (const CanonAddr& addr, const Hash& hash);
    Hash computeAreaPartialHash (const CanonAddr& beginAddr, size_t size);
};

}}

#endif 
