// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

#ifndef MM_RUNTIMEVISITOR_H
#define MM_RUNTIMEVISITOR_H


namespace mmodule {

/**
 * Listener interface that is used to report problems when accessing the memory.
 */
struct MemoryAccessListener {
    virtual ~MemoryAccessListener () {}
    
    /**
     * This function is called exactly once for every unreachable memory area.
     * Memory area is unreachable when there is no sequence of memory areas a_1, a_2, ..., a_n 
     * such that a_1 is the root memory area and a_i contains pointer 
     * to a_{i+1} for i = 1, ..., n-1.
     * @warning The current state of the memory must not be modified from the body 
     * of this method.
     * 
     * @param areaId id of the unreachable area
     * @see MemoryArea::getRootArea()
     */
    virtual void unreachableArea (unsigned int areaId) = 0;
};

}

#endif
