// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

#include "Ptr.h"

#include "AreaEntry.h"


namespace mmodule {
    
/**
 * Returns the size of the area this pointer points to.
 * 
 * @return size of the area this pointer points to in bytes
 */
size_t 
Ptr::getAreaSize () const
{ 
    return m_area->getSize (); 
}


/**
 * Checks if the area this pointer points to is deleted.
 *
 * @return @c true if the area is deleted, @c false otherwise
 */
bool
Ptr::isAreaDeleted () const
{
    return m_area->isDeleted ();
}

/**
 * Returns ID of the area part of this pointer.
 *
 * @return ID of the area part of this pointer
 */
size_t
Ptr::getAreaId () const
{
    return m_area->getAreaId();
}

/**
 * Checks if the area this pointer points to is exapndable.
 *
 * @return @c true if the area is expandable, @c false otherwise
 */
bool
Ptr::isAreaExpandable () const
{
	return m_area->isExpandable();
}

}
