// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

#ifndef MM_RANDOMVALUES_H
#define MM_RANDOMVALUES_H

#include "CachingFunctor.h"
#include "Types.h"

#include <boost/random/linear_congruential.hpp>
#include <boost/random/uniform_int.hpp>
#include <boost/random/variate_generator.hpp>


namespace mmodule { namespace detail {

/**
 * Functor returning random values for canonical addresses.
 */
class RandomValues {
    /**
     * Functor generating the random values.
     */
    class Generator {
        typedef boost::minstd_rand Gen_;
        typedef boost::uniform_int<Hash> UnifInt_;
        typedef boost::variate_generator<Gen_&, UnifInt_> VarGen_;
        
        Gen_ m_generator;
        VarGen_ m_varGen;
        
      public:
        typedef CanonAddr Arg;
        typedef Hash Result;
        
        Generator () : m_generator (13u), m_varGen (m_generator, UnifInt_ (0, hashModul ()))
        {
        }
        
        /**
         * Returns random value from range <tt>&lt;0, hashModul())</tt>.
         * 
         * @return random value from range <tt>&lt;0, hashModul())</tt>
         */
        Hash operator() (const CanonAddr&) 
        {
            return m_varGen();
        }
    };
    
    Generator m_generator;
    CachingFunctor<Generator> m_func;
    
  public:
    RandomValues () : m_generator (), m_func (m_generator) {}

    /**
     * Returns random value for canonical address @c addr.
     * Multiple calls with the same argument return the same values.
     * 
     * @param addr canonical address for which to return the random value
     * @return random value for canonical address @c addr
     */
    Hash operator() (const CanonAddr& addr)
    {
        return m_func (addr);
    }
};

}}

#endif
