// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

#ifndef MM_REF_H
#define MM_REF_H

#include "AreaEntry.h"

namespace mmodule {
    
namespace detail {
class AreaEntry;
}


/**
 * Class representing reference to memory area that can store only members
 * of structure T.
 * 
 * @tparam T POD structure
 * 
 * @see Ptr
 */
template <typename T>
class Ref {
  private:
    /**
     * area entry representing the memory area this object references to
     */
    detail::AreaEntry* m_area;
    
  public:
    /**
     * Creates null reference.
     * 
     * @tparam T POD structure
     * @warning
     *      there is no way how to check, that @c T is really a POD structure, 
     *      on compilation time, 
     *      so if @c T is not a POD structure, some strange compilation errors might occur, 
     *      or if everything compiles then the behaviour is undefined
     */
    Ref () : m_area (0) {}
    
    /**
     * Checks if this is null reference.
     * 
     * @return @c true if this object is null reference, @c false otherwise
     */
    bool isNull () const
    { return m_area == 0; }
    
    /**
     * Operator ==.
     */
    bool operator== (const Ref<T>& ref) const
    { return m_area == ref.m_area; }
    
    /**
     * Operator !=.
     */
    bool operator!= (const Ref<T>& ref) const
    { return !(*this == ref); }
    
    
  protected:
    /**
     * Constructor.
     * 
     * @param area area entry representing the memory area the newly created object
     *      should reference to
     */
    Ref (detail::AreaEntry* area) : m_area (area) {}
    
    /**
     * Returns the area entry representing the memory area this object references to.
     * 
     * @return the area entry representing the memory area this object references to
     */
    detail::AreaEntry* getAreaEntry () const
    { return m_area; }
    
    friend class MemoryState;
    template <typename V> friend class detail::Content_;
};

}

#endif
