// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

#include "RefContentEntry.h"

#include "AreaEntry.h"
#include "Content.h"
#include "../Error.h"

#include <typeinfo>


namespace mmodule { namespace detail {

/**
 * Constructor.
 * 
 * @param source memory area this content belongs to
 * @param offset offset in the area where is this content stored
 * @param content pointer content
 */
RefContentEntry::RefContentEntry (AreaEntry* source,
                                  size_t offset, std::auto_ptr<PtrContent> content)
    : ContentEntry (offset, std::auto_ptr<Content> (content)), 
      m_source (source), m_isInState (false)
{
}


/**
 * Destructor.
 */
RefContentEntry::~RefContentEntry ()
{
}


/**
 * Returns target of the pointer stored in this object.
 * 
 * @return target of the pointer stored in this object
 */
AreaEntry* 
RefContentEntry::getTarget () const
{
    return getPtrContent ()->getPtr ().getTargetArea ();
}


/**
 * Adds this link into the outgoing links set of this link's source and 
 * into the incomming links list of this link's target.
 * 
 * Complexity: O(log(m_source->outLinksCount ())).
 */
void 
RefContentEntry::addLink ()
{
    if (m_isInState)
        return;
    
    if (!getTarget ())
        return;
    
    LOG_DEBUG ("adding link   %p:%4zu->%p", m_source, getOffset (),
               getTarget ());
    
    m_source->addOutLink (this);
    getTarget ()->addInLink (this);
    
    m_isInState = true;
}
    

/**
 * Reverse operation to @c addLink().
 * 
 * Complexity: Average O(1).
 */
void 
RefContentEntry::removeLink ()
{
    if (!m_isInState)
        return;
    
    if (!getTarget ())
        return;
    
    LOG_DEBUG ("removing link %p:%4zu->%p", m_source, getOffset (),
               getTarget ());

    m_source->removeOutLink (this);
    getTarget ()->removeInLink (this);
    
    m_isInState = false;
}

}}
