// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

#ifndef MM_REFCONTENTENTRY_H
#define MM_REFCONTENTENTRY_H

#include "ContentEntry.h"

#include <boost/intrusive/list.hpp>
#include <boost/intrusive/set.hpp>
#include <cassert>


namespace mmodule { namespace detail {
    
struct InLinksTag {};
typedef boost::intrusive::list_base_hook<
        boost::intrusive::optimize_size<true>,
        boost::intrusive::tag<InLinksTag> > InLinksBaseHook;
        
struct OutLinksTag {};
typedef boost::intrusive::set_base_hook<
        boost::intrusive::optimize_size<true>,
        boost::intrusive::tag<OutLinksTag> > OutLinksBaseHook;

        
/**
 * Class holding pointer value stored in the memory.
 * This class represents a link between the memory area this entry belongs to and
 * the area the content stored in this entry points to.
 */
class RefContentEntry 
    : public ContentEntry,
      public InLinksBaseHook, public OutLinksBaseHook
{
    /*! area to which this content belongs to */
    AreaEntry* m_source;
    
    /*! @c true if the link represented by this object was added into the state of the memory */
    bool m_isInState : 1;
    
    /*! used internaly in @c RelationsUpdater */
    bool m_relationUpdateNeeded : 1;
    
  public:
    RefContentEntry (AreaEntry* source, size_t offset, std::auto_ptr<PtrContent> content);
    ~RefContentEntry ();
    
    /**
     * Returns area to which this content belongs to.
     * 
     * @return area to which this content belongs to
     */
    AreaEntry* getSource () const
    { return m_source; }
    
    AreaEntry* getTarget () const;
    
    /**
     * Returns pointer content this object holds.
     * 
     * @return pointer content this object holds
     */
    const PtrContent* getPtrContent () const
    { 
        const PtrContent* ptrC = dynamic_cast<const PtrContent*> (getContent ()); 
        assert (ptrC);
        return ptrC;
    }
        
    /**
     * Checks if the link represented by this object is valid.
     * 
     * @return @c true if the link represented by this object is valid, @c false otherwise
     */
    bool isValidLink () const
    { return getTarget () != 0; }
    
    /*! used internaly in @c RelationsUpdater */
    bool isRelationUpdateNeeded () const
    { return m_relationUpdateNeeded; }
    
    /*! used internaly in @c RelationsUpdater */
    void setRelationUpdateNeeded (bool relationUpdateNeeded) 
    { m_relationUpdateNeeded = relationUpdateNeeded; }
    
    
    void addLink ();
    void removeLink ();
};

}}

#endif
