// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

#ifndef UNDO_H
#define	UNDO_H

#include <boost/noncopyable.hpp>
#include <boost/intrusive/slist.hpp>

namespace mmodule { namespace detail {
class State;

/**
 * Abstract class representing an undoable change of the memory
 * state. The instances of @c Undo are linked into singly-linked undo lists,
 * so that they can be unrolled together during backtracking.
 */
class Undo : private boost::noncopyable, public boost::intrusive::slist_base_hook<> {
public:
    virtual ~Undo();

    virtual void undo(State& state) = 0;
protected:
    Undo(){}
};

}}

#endif	/* UNDO_H */

