// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

#include <boost/checked_delete.hpp>

#include "../../Error.h"
#include "UndoSet.h"

using namespace mmodule::detail;

UndoSet::UndoSet()
{
}

UndoSet::~UndoSet()
{
    LOG_DEBUG ("Disposing UndoSet of size: %zu", m_undos.size());
    m_undos.reverse ();
    m_undos.clear_and_dispose(boost::checked_delete<Undo>);
}

void
UndoSet::addUndo(Undo* undo)
{
    m_undos.push_front(*undo);
    LOG_DEBUG ("Added an undo, total %zu", m_undos.size());
}

void
UndoSet::undo(State& state)
{
    LOG_DEBUG ("Undoing UndoSet of size: %zu", m_undos.size());

    Undos::iterator it;
    Undos::iterator end = m_undos.end();

    for (it = m_undos.begin(); it != end; it++) {
        it->undo(state);
    }
}
