// 
// GMC Gimple Model Checker
// Copyright (C) 2009-2011 GMC team
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of 
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public 
// License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.

#ifndef UNDOSET_H
#define	UNDOSET_H

#include <boost/noncopyable.hpp>
#include <boost/intrusive/slist.hpp>

#include "Undo.h"
#include "../Types.h"

namespace mmodule { namespace detail {

class UndoSet : public Undo {
public:
    UndoSet();
    virtual ~UndoSet();

    void addUndo(Undo* undo);

    virtual void undo(State& state);
    
    /**
     * Returns hash of the state of the memory right after the modifications,
     * stored in this object, were applied.
     *
     * @return hash of the state of the memory right after the modifications,
     *      stored in this object, were applied
     * @see setHash()
     */
    Hash getHash () const
    { return m_hash; }

    /**
     * @see getHash()
     */
    void setHash (Hash hash)
    { m_hash = hash; }

private:
    typedef boost::intrusive::slist<Undo> Undos;

    /**
     * List of @c Undo that represents all the changes in the order oposite
     * to the order in which the changes have been made. All the instances
     * of @c Undo are owned by the list and deallocated when this class
     * is destroyed.
     */
    Undos m_undos;

    /**
     * hash of the state of the memory right after the modifications, stored in this object,
     * were applied
     */
    Hash m_hash;
};

}}

#endif	/* UNDOSET_H */

