#!/bin/bash

GCC_VERSION="4.5.0"
GCC_DIR="gcc-${GCC_VERSION}"
GCC_FILE="gcc-${GCC_VERSION}.tar.gz"
SERVER_PATH="ftp://ftp.gnu.org/gnu/gcc"
SERVER_FILE="${SERVER_PATH}/${GCC_DIR}/${GCC_FILE}"

PATCH="dump-gimplexx-patch-gcc-${GCC_VERSION}.diff"

check_result(){
    if [ $? -ne 0 ]; then
        echo "... failed"
        exit 1
    else
        echo "... OK"
    fi
}

# Check prior existence of the GCC source directory
if [ -d "${GCC_DIR}" ]; then
    echo "The directory with GCC sources already exists! Remove it before running this script."
    exit 1;
fi

# Get the GCC sources package
if [ -f "${GCC_FILE}" ]; then
    echo "Found source package ${GCC_FILE}, skipping download. To force download, delete the file."
else
    echo -n "Downloading GCC sources from ${SERVER_FILE}"
    wget ${SERVER_FILE}
    check_result
fi

# Unpack them
echo -n "Unpacking GCC sources ${GCC_FILE}"
tar xzf "${GCC_FILE}"
check_result

# Patch the sources
cd "${GCC_DIR}"
echo -n "Applying gimplexx serialization patch ${PATCH}"
patch -s -p1 < "../patches/${PATCH}"
check_result
